@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 
 __author__ = "Rob Knight, Greg Caporaso"
-__copyright__ = "Copyright 2010, The QIIME Project"
+__copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Rob Knight", "Greg Caporaso", "Kyle Bittinger", "Antonio Gonzalez Pena"]
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -19,14 +19,15 @@ from itertools import count
 from string import strip
 from shutil import copy as copy_file
 from tempfile import NamedTemporaryFile
+from cStringIO import StringIO
 from cogent import LoadSeqs, DNA
-from cogent.app.util import get_tmp_filename
+from qiime.util import get_tmp_filename
 from cogent.app.formatdb import build_blast_db_from_fasta_path
 from cogent.app.blast import blast_seqs, Blastall, BlastResult
-from cogent.app.rdp_classifier import assign_taxonomy, \
-    train_rdp_classifier_and_assign_taxonomy
+import qiime.pycogent_backports.rdp_classifier
+import cogent.app.rdp_classifier20
 from cogent.parse.fasta import MinimalFastaParser
-from qiime.util import FunctionWithParams
+from qiime.util import FunctionWithParams, get_rdp_jarpath
 
 
 """Contains code for assigning taxonomy, using several techniques.
@@ -34,6 +35,48 @@ from qiime.util import FunctionWithParams
 This module has the responsibility for taking a set of sequences and
 providing a taxon assignment for each sequence."""
 
+def check_rdp_version(rdp_jar_path,requested_version):
+    return requested_version in rdp_jar_path
+
+def error_on_bad_rdp_version(option_parser,assignment_method):
+        rdp_jarpath = get_rdp_jarpath()
+        if rdp_jarpath == None:
+            option_parser.error("RDP classifier is not installed or "
+             "not accessible to QIIME. See install instructions here: "
+             "http://qiime.org/install/install.html#rdp-install")
+        elif assignment_method == 'rdp':
+            if not check_rdp_version(rdp_jarpath,"2.2"):
+                option_parser.error("Specified RDP version 2.2 (default), "
+                "but that version is not installed. Pass -m rdp20 for RDP "
+                "classifier versions 2.0 and 2.0.1.")
+        elif assignment_method == 'rdp20':
+            if not check_rdp_version(rdp_jarpath,"2.0"):
+                option_parser.error("Specified RDP version 2.0 (default), "
+                "but that version is not installed. Pass -m rdp for RDP "
+                "classifier versions 2.2.")
+        else:
+            # not possible to get here, but don't like if/elif without else
+            raise ValueError,\
+             "Unknown assignment method: %s" % assignment_method
+
+def guess_rdp_version():
+    rdp_jarpath = get_rdp_jarpath()
+    if rdp_jarpath == None:
+        raise ValueError, \
+         ("RDP classifier is not installed or "
+          "not accessible to QIIME. See install instructions here: "
+          "http://qiime.org/install/install.html#rdp-install")
+    elif "2.2" in rdp_jarpath:
+        return "rdp22"
+    elif "2.0" in rdp_jarpath:
+        return "rdp20"
+    else:
+        raise ValueError,\
+         ("Can't determine RDP version number. Only versions 2.0, 2.0.1,"
+          " and 2.2 are supported by QIIME. RDP jar path is:"
+          " %s" % rdp_jarpath)
+    
+
 class TaxonAssigner(FunctionWithParams):
     """A TaxonAssigner assigns a taxon to each of a set of sequences.
 
@@ -288,11 +331,11 @@ class RdpTaxonAssigner(TaxonAssigner):
     """Assign taxon using RDP's naive Bayesian classifier
     """
     Name = "RdpTaxonAssigner"
-    Application = "RDP classfier"
+    Application = "RDP classfier, version 2.2"
     Citation = "Wang, Q, G. M. Garrity, J. M. Tiedje, and J. R. Cole. 2007. Naive Bayesian Classifier for Rapid Assignment of rRNA Sequences into the New Bacterial Taxonomy. Appl Environ Microbiol. 73(16):5261-7."
     Taxonomy = "RDP"
     _tracked_properties = ['Application','Citation','Taxonomy']
-
+    
     def __init__(self, params):
         """Return new RdpTaxonAssigner object with specified params.
         
@@ -309,6 +352,14 @@ class RdpTaxonAssigner(TaxonAssigner):
         _params.update(params)
         TaxonAssigner.__init__(self, _params)
 
+    @property
+    def _assign_fcn(self):
+        return cogent.app.rdp_classifier.assign_taxonomy
+
+    @property
+    def _train_fcn(self):
+        return cogent.app.rdp_classifier.train_rdp_classifier_and_assign_taxonomy
+
     def __call__(self, seq_path, result_path=None, log_path=None):
         """Returns dict mapping {seq_id:(taxonomy, confidence)} for
         each seq.
@@ -320,24 +371,28 @@ class RdpTaxonAssigner(TaxonAssigner):
         log_path: path to log, which should include dump of params.
         """
         
-        min_confidence = self.Params['Confidence']
+        min_conf = self.Params['Confidence']
         training_data_properties_fp = self.Params['training_data_properties_fp']
         reference_sequences_fp = self.Params['reference_sequences_fp']
         id_to_taxonomy_fp = self.Params['id_to_taxonomy_fp']
         
         seq_file = open(seq_path, 'r')
-
         if reference_sequences_fp and id_to_taxonomy_fp:
             # Train and assign taxonomy
             taxonomy_file, training_seqs_file = self._generate_training_files()
-            results = train_rdp_classifier_and_assign_taxonomy(
+            results = self._train_fcn(
                 training_seqs_file, taxonomy_file, seq_file, 
-                min_confidence=min_confidence,
+                min_confidence=min_conf,
                 classification_output_fp=result_path)
+
+            if result_path is None:
+                results = self._training_set.fix_results(results)
+            else:
+                self._training_set.fix_output_file(result_path)
         else:
             # Just assign taxonomy, using properties file if passed
-            results = assign_taxonomy(
-                seq_file, min_confidence=min_confidence, output_fp=result_path,
+            results = self._assign_fcn(
+                seq_file, min_confidence=min_conf, output_fp=result_path,
                 training_data_fp=training_data_properties_fp)
 
         if log_path:
@@ -349,35 +404,69 @@ class RdpTaxonAssigner(TaxonAssigner):
         """Returns a tuple of file objects suitable for passing to the
         RdpTrainer application controller.
         """
-        id_to_taxonomy_file = open(self.Params['id_to_taxonomy_fp'], 'r')
+        training_set = RdpTrainingSet()
         reference_seqs_file = open(self.Params['reference_sequences_fp'], 'r')
+        id_to_taxonomy_file = open(self.Params['id_to_taxonomy_fp'], 'r')
 
-        # Generate taxonomic tree and write to file 
-        tree = self._build_tree(id_to_taxonomy_file)
-        id_to_taxonomy_file.seek(0)
+        for seq_id, seq in MinimalFastaParser(reference_seqs_file):
+            training_set.add_sequence(seq_id, seq)
+
+        for line in id_to_taxonomy_file:
+            seq_id, lineage_str = map(strip, line.split('\t'))
+            training_set.add_lineage(seq_id, lineage_str)
+
+        training_set.dereplicate_taxa()
 
         rdp_taxonomy_file = NamedTemporaryFile(
             prefix='RdpTaxonAssigner_taxonomy_', suffix='.txt')
-        rdp_taxonomy_file.write(tree.rdp_taxonomy())
+        rdp_taxonomy_file.write(training_set.get_rdp_taxonomy())
         rdp_taxonomy_file.seek(0)
 
-        # Generate a set of training seqs and write to file
-        training_seqs = self._generate_training_seqs(
-            reference_seqs_file, id_to_taxonomy_file)
-
         rdp_training_seqs_file = NamedTemporaryFile(
             prefix='RdpTaxonAssigner_training_seqs_', suffix='.fasta')
-        for rdp_id, seq in training_seqs:
+        for rdp_id, seq in training_set.get_training_seqs():
             rdp_training_seqs_file.write('>%s\n%s\n' % (rdp_id, seq))
         rdp_training_seqs_file.seek(0)
 
+        self._training_set = training_set
+
         return rdp_taxonomy_file, rdp_training_seqs_file
 
-    @staticmethod
-    def _generate_training_seqs(reference_seqs_file, id_to_taxonomy_file):
+
+class RdpTrainingSet(object):
+    def __init__(self):
+        self._tree = RdpTree()
+        self.sequences = {}
+        self.sequence_nodes = {}
+
+    def add_sequence(self, seq_id, seq):
+        self.sequences[seq_id] = seq
+
+    def add_lineage(self, seq_id, lineage_str):
+        lineage = self._parse_lineage(lineage_str)
+        seq_node = self._tree.insert_lineage(lineage)
+        self.sequence_nodes[seq_id] = seq_node
+
+    def dereplicate_taxa(self):
+        return self._tree.dereplicate_taxa()
+
+    def _parse_lineage(self, lineage_str):
+        """Returns a list of taxa from the semi-colon-separated
+        lineage string of an id_to_taxonomy file.
+        """
+        lineage = lineage_str.strip().split(';')
+        if len(lineage) != 6:
+            raise ValueError(
+                'Each reference assignment must contain 6 items, specifying '
+                'domain, phylum, class, order, family, and genus.  '
+                'Detected %s items in "%s": %s.' % \
+                (len(lineage), lineage_str, lineage))
+        return lineage
+
+    def get_training_seqs(self):
         """Returns an iterator of valid training sequences in
         RDP-compatible format
-
+        
         Each training sequence is represented by a tuple (rdp_id,
         seq).  The rdp_id consists of two items: the original sequence
         ID with whitespace replaced by underscores, and the lineage
@@ -387,117 +476,133 @@ class RdpTaxonAssigner(TaxonAssigner):
         # trust user IDs to not have whitespace, so we replace all
         # whitespace with an underscore.  Classification may fail if
         # the replacement method generates a name collision.
+        for seq_id, node in self.sequence_nodes.iteritems():
+            seq = self.sequences.get(seq_id)
+            if seq is not None:
+                lineage = node.get_lineage()
+                rdp_id = '%s %s' % (re.sub('\s', '_', seq_id), ';'.join(lineage))
+                yield rdp_id, seq
 
-        id_to_taxonomy_map = RdpTaxonAssigner._parse_id_to_taxonomy_file(
-            id_to_taxonomy_file)
+    def get_rdp_taxonomy(self):
+        return self._tree.get_rdp_taxonomy()
 
-        for id, seq in MinimalFastaParser(reference_seqs_file):
-            taxonomy = id_to_taxonomy_map[id]
-            lineage = RdpTaxonAssigner._parse_lineage(taxonomy)
-            rdp_id = '%s %s' % (re.sub('\s', '_', id), ';'.join(lineage))
-            yield rdp_id, seq
+    def fix_output_file(self, result_path):
+        # Ultimate hack to replace mangled taxa names
+        temp_results = StringIO()
+        for line in open(result_path):
+            untagged_line = re.sub(
+                _QIIME_RDP_TAXON_TAG + "[^;\n\t]*", '', line)
+            temp_results.write(untagged_line)
+        open(result_path, 'w').write(temp_results.getvalue())
 
-    @staticmethod
-    def _build_tree(id_to_taxonomy_file):
-        """Returns an RdpTree object representing the taxonomic tree
-        derived from the id_to_taxonomy file.
+    def fix_results(self, results_dict):
+        for seq_id, assignment in results_dict.iteritems():
+            lineage, confidence = assignment
+            revised_lineage = re.sub(
+                _QIIME_RDP_TAXON_TAG + "[^;\n\t]*", '', lineage)
+            results_dict[seq_id] = (revised_lineage, confidence)
+        return results_dict
+
+
+class RdpTree(object):
+    """Simple, specialized tree class used to generate a taxonomy
+    file for the Rdp Classifier.
+    """
+    taxonomic_ranks = [
+        'norank', 'domain', 'phylum', 'class', 'order', 'family', 'genus']
+
+    def __init__(self, name='Root', parent=None, counter=None):
+        if counter is None:
+            self.counter = count(0)
+        else:
+            self.counter = counter
+        self.id = self.counter.next()
+        self.name = name
+        self.parent = parent
+        self.seq_ids = []
+        if parent is None:
+            self.depth = 0
+        else:
+            self.depth = parent.depth + 1
+        self.children = dict()  # name => subtree
+
+    def insert_lineage(self, lineage):
+        """Inserts an assignment into the taxonomic tree.
+        
+        Lineage must support the iterator interface, or provide an
+        __iter__() method that returns an iterator.
         """
-        tree = RdpTaxonAssigner.RdpTree()
-        for line in id_to_taxonomy_file:
-            id, taxonomy = map(strip, line.split('\t'))
-            lineage = RdpTaxonAssigner._parse_lineage(taxonomy)
-            tree.insert_lineage(lineage)
-        return tree
+        lineage = iter(lineage)
+        try:
+            taxon = lineage.next()
+            if taxon not in self.children:
+                self.children[taxon] = self.__class__(
+                    name=taxon, parent=self, counter=self.counter)
+            retval = self.children[taxon].insert_lineage(lineage)            
+        except StopIteration:
+            retval = self
+        return retval
 
-    @staticmethod
-    def _parse_lineage(lineage_str):
-        """Returns a list of taxa from the semi-colon-separated
-        lineage string of an id_to_taxonomy file.
-        """
-        lineage = lineage_str.strip().split(';')
-        # The RDP Classifier can only deal with a lineage that is 6
-        # levels deep.  We detect this problem now to avoid an
-        # ApplicationError later on.
-        if len(lineage) != 6:
-            raise ValueError(
-                'Each reference assignment must contain 6 items, specifying '
-                'domain, phylum, class, order, family, and genus.  '
-                'Detected %s items in "%s": %s.' % \
-                (len(lineage), lineage_str, lineage))
-        return lineage
+    def get_lineage(self):
+        if self.parent is not None:
+            return self.parent.get_lineage() + [self.name]
+        else:
+            return [self.name]
 
-    # The RdpTree class is defined as a nested class to prevent the
-    # implementation details of creating RDP-compatible training files
-    # from being exposed at the module level.
-    class RdpTree(object):
-        """Simple, specialized tree class used to generate a taxonomy
-        file for the Rdp Classifier.
-        """
-        taxonomic_ranks = ['domain', 'phylum', 'class', 
-                           'order', 'family', 'genus']
+    def get_nodes(self):
+        yield self
+        for child in self.children.values():
+            child_nodes = child.get_nodes()
+            for node in child_nodes:
+                yield node
 
-        def __init__(self, name='Root', parent=None):
-            self.name = name
-            self.parent = parent
-            if parent is None:
-                self.depth = 0
+    def dereplicate_taxa(self):
+        taxa_by_depth = {}
+        for node in self.get_nodes():
+            name = node.name
+            depth = node.depth
+            current_names = taxa_by_depth.get(depth, set())
+            if name in current_names:
+                node.name = name + _QIIME_RDP_TAXON_TAG + str(node.id)
             else:
-                self.depth = parent.depth + 1
-            self.children = dict()  # name => subtree
+                current_names.add(name)
+                taxa_by_depth[depth] = current_names
 
-        def insert_lineage(self, lineage):
-            """Inserts a lineage into the taxonomic tree.
-            
-            Lineage must support the iterator interface, or provide an
-            __iter__() method that returns an iterator.
-            """
-            lineage = lineage.__iter__()
-            try:
-                taxon = lineage.next()
-                if taxon not in self.children:
-                    self.children[taxon] = RdpTaxonAssigner.RdpTree(name=taxon, parent=self)
-                self.children[taxon].insert_lineage(lineage)
-            except StopIteration:
-                pass
-            return self
+    def get_rdp_taxonomy(self):
+        """Returns a string, in Rdp-compatible format.
+        """
+        # RDP uses 0 for the parent ID of the root node
+        if self.parent is None:
+            parent_id = 0
+        else:
+            parent_id = self.parent.id
 
-        def rdp_taxonomy(self, counter=None):
-            """Returns a string, in Rdp-compatible format.
-            """
-            if counter is None:
-                # initialize a new counter to assign IDs
-                counter = count(0)
+        rank_name = self.taxonomic_ranks[self.depth]
 
-            # Assign ID to current node; used by child nodes
-            self.id = counter.next()
+        fields = [
+            self.id, self.name, parent_id, self.depth, rank_name]
+        taxonomy_str = '*'.join(map(str, fields)) + "\n"
 
-            if self.parent is None:
-                # do not print line for Root node
-                retval = ''
-            else:
-                # Rdp taxonomy file does not count the root node
-                # when considering the depth of the taxon.  We
-                # therefore specify a taxon depth which is one
-                # less than the tree depth.
-                taxon_depth = self.depth - 1
+        # Recursively append lines from sorted list of subtrees
+        child_names = self.children.keys()
+        child_names.sort()
+        subtrees = [self.children[name] for name in child_names]
+        for subtree in subtrees:
+            taxonomy_str += subtree.get_rdp_taxonomy()
+        return taxonomy_str
 
-                # In this simplest-possible implementation, we
-                # also use the taxon depth to retrieve the string
-                # value of the taxonomic rank.  Taxa beyond the
-                # depth of our master list are given a rank of ''.
-                try:
-                    taxon_rank = self.taxonomic_ranks[taxon_depth]
-                except IndexError:
-                    taxon_rank = ''
 
-                fields = [self.id, self.name, self.parent.id, taxon_depth,
-                          taxon_rank]
-                retval = '*'.join(map(str, fields)) + "\n"
+class Rdp20TaxonAssigner(RdpTaxonAssigner):
+    Name = "Rdp20TaxonAssigner"
+    Application = "RDP classfier, version 2.0"
+    
+    @property
+    def _assign_fcn(self):
+        return cogent.app.rdp_classifier20.assign_taxonomy
 
-            # Recursively append lines from sorted list of subtrees
-            child_names = self.children.keys()
-            child_names.sort()
-            subtrees = [self.children[name] for name in child_names]
-            for subtree in subtrees:
-                retval += subtree.rdp_taxonomy(counter)
-            return retval
+    @property
+    def _train_fcn(self):
+        return cogent.app.rdp_classifier20.train_rdp_classifier_and_assign_taxonomy
+
+
+_QIIME_RDP_TAXON_TAG = "_qiime_unique_taxon_tag_"
