@@ -2,10 +2,10 @@
 #file colors.py
 
 __author__ = "Jesse Stombaugh"
-__copyright__ = "Copyright 2010, The QIIME Project" #consider project name
+__copyright__ = "Copyright 2011, The QIIME Project" #consider project name
 __credits__ = ["Rob Knight","Jesse Stombaugh"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Jesse Stombaugh"
 __email__ = "jesse.stombaugh@colorado.edu"
 __status__ = "Release"
@@ -552,7 +552,7 @@ def taxonomy_color_prefs_and_map_data_from_options(options):
     #need to set some other way from sample ids
     #Determine which mapping headers to color by, if none given, color by \
     #Sample ID's
-    taxonomy_count_files = options.counts_fname.strip().strip("'").split(',')
+    taxonomy_count_files = options.counts_fname
     for f in taxonomy_count_files:
         try:
             counts_f = open(f, 'U').readlines()
