@@ -1,18 +1,18 @@
 #!/usr/bin/env python
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Rob Knight", "Justin Kuczynski","Jeremy Widmann", \
         "Antonio Gonzalez Pena", "Daniel McDonald"] 
 #remember to add yourself if you make changes
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
 
 import numpy
-from numpy import isnan
+from numpy import isnan, log10, median
 from StringIO import StringIO
 from cogent import Sequence
 
@@ -21,6 +21,63 @@ from cogent import Sequence
 A lot of this might migrate into cogent at some point.
 """
 
+def format_mapping_file(headers, mapping_data, comments=None):
+    """ returns a large formatted string representing the entire mapping file
+
+    each input is a list, and all data should be strings, not e.g. ints
+    * headers defines column labels, and SampleID should not include a '#'
+    * mapping_data is a list of lists, each sublist is a row in the mapping file
+    each mapping_data sublist must be the same length as headers - use ''
+    for absent data
+    * if included, commments will be inserted above the header line
+    comments should not include a # - that will be appended in this formatter
+    """
+
+    result = [] # each elem is a string representing a line
+
+    result.append('#' + '\t'.join(headers))
+
+    if comments != None:
+        for comment in comments:
+            result.append('#' + comment)
+
+    for mapping_line in mapping_data:
+        if not (len(mapping_line) == len(headers)):
+            raise RuntimeError('error formatting mapping file, does each '+\
+             'sample have the same length of data as the headers?')
+        result.append('\t'.join(mapping_line))
+
+    str_result = '\n'.join(result)
+    return str_result
+
+def format_p_value_for_num_iters(p,num_iters):
+    """adjust p to str w correct num of decimals for num monte carlo iters
+    """
+    if num_iters < 10:
+        # this can be the last step of a long process, so we don't 
+        # want to fail
+        return "Too few iters to compute p-value (num_iters=%d)" % num_iters
+    decimal_places = int(log10(num_iters))
+    result = ('%1.'+'%df' % decimal_places) % p
+    return result
+
+def format_qiime_parameters(params, header="#QIIME parameters"):
+    """Formats lines for qiime_parameters.txt"""
+    qiime_params = [header]
+    for script, options in sorted(params.items()):
+        for option, value in sorted(options.items()):
+            specific_option = ':'.join([script, option])
+            
+            # Based on how qiime_parameters is parsed
+            if value is None:
+                value = "True"
+
+            # cast value to string just in case
+            full_line = '\t'.join([specific_option, str(value)])
+
+            qiime_params.append(full_line)
+    return qiime_params
+
 def format_summarize_taxa(summary, header, delimiter=';'):
     """Formats a summarized taxonomy table for output"""
     yield "%s\n" % '\t'.join(header)
@@ -204,8 +261,8 @@ def format_nmds_coords(samples, points, stress):
     """ samples is list, points is samples by axis coord (typ many by 2 mtx)
     """
     result = []
-    result.append('nmds\t' +
-        '\t'.join(map(str, range(1,points.shape[1]+1))))
+    col_headers = ["NMDS"+str(aa) for aa in range(1,points.shape[1]+1)]
+    result.append('sample\t' + '\t'.join(col_headers) )
     for name, row in zip(samples, points):
         result.append('\t'.join([name] + map(str, row)))
     result.append('')
@@ -392,6 +449,42 @@ def format_histograms(pre_hist, post_hist, bin_edges):
     for edge, pre, post in zip(bin_edges, pre_hist, post_hist):
         lines.append('\t'.join(map(str, [edge, pre, post])))
     return '\n'.join(lines)
+    
+def format_histogram_one_count(counts, bin_edges):
+    """Returns text-formatted histogram with only one count."""
+    lines = []
+    lines.append('Length\tCount')
+    for edge, count in zip(bin_edges, counts):
+        lines.append('\t'.join(map(str, [edge, count])))
+    return '\n'.join(lines)
+    
+def format_split_libraries_fastq_log(count_barcode_not_in_map,
+               count_too_short,
+               count_too_many_N,
+               count_bad_illumina_qual_digit,
+               count_barcode_errors_exceed_max,
+               input_sequence_count,
+               sequence_lengths,
+               seqs_per_sample_counts):
+    """ Format the split libraries log """
+    log_out = ["Quality filter results"]
+    log_out.append("Total number of input sequences: %d" % input_sequence_count)
+    log_out.append("Barcode not in mapping file: %d" % count_barcode_not_in_map)
+    log_out.append("Read too short after quality truncation: %d" % count_too_short)
+    log_out.append("Count of N characters exceeds limit: %d" % count_too_many_N)
+    log_out.append("Illumina quality digit = 0: %d" % count_bad_illumina_qual_digit)
+    log_out.append("Barcode errors exceed max: %d" % count_barcode_errors_exceed_max)
+    
+    log_out.append("")
+    
+    log_out.append("Result summary (after quality filtering)")
+    log_out.append("Median sequence length: %1.2f" % median(sequence_lengths))
+    counts = [(v,k) for k,v in seqs_per_sample_counts.items()]
+    counts.sort()
+    counts.reverse()
+    for sequence_count, sample_id in counts:
+        log_out.append('%s\t%d' % (sample_id,sequence_count))
+    return '\n'.join(log_out)
 
 def format_unifrac_sample_mapping(sample_ids, otu_ids, otu_table_array):
     """Returns a unifrac sample mapping file from output of parse_otu_table
@@ -415,5 +508,45 @@ def write_Fasta_from_name_seq_pairs(name_seqs, fh):
 
     for (name,seq) in name_seqs:
         fh.write("%s\n"% Sequence(name=name, seq = seq).toFasta())
+        
+def illumina_data_to_fastq(record_data,number_of_bases=None):
+    """ given data from an Illumina qseq file, write to fastq 
     
+        read data: generator of tuples with the following data
+         (machine ID, 
+          ... [GREG: NEED TO FILL IN DETAILS!]
+          illumina quality digit (0:failed screen; 1: passed screen)
+          read number,
+          sequence,
+          quality string)
+          
+        number_of_bases: number of bases to keep, starting from
+         beginnng of the read - useful when additional cycles are
+         applied (e.g., sometimes happens when sequencing barcodes)
+    
+    """
+    seq_index = 8
+    qual_index = 9
+    if number_of_bases == None:
+        seq = record_data[seq_index].replace('.','N')
+        qual = record_data[qual_index]
+    else:
+        seq = record_data[seq_index][:number_of_bases].replace('.','N')
+        qual = record_data[qual_index][:number_of_bases]
+    
+    
+    header = '%s_%s:%s:%s:%s:%s#%s/%s' % (
+      record_data[0],
+      record_data[1],
+      record_data[2],
+      record_data[3],
+      record_data[4],
+      record_data[5],
+      record_data[6],
+      record_data[7])
+    
+    return '@%s\n%s\n+\n%s' % (header,
+      seq,
+      qual)
+
 
