@@ -2,11 +2,11 @@
 #file make_3d_plots.py
 
 __author__ = "Jesse Stombaugh, Rob Knight, and Dan Knights"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Jesse Stombaugh", "Rob Knight", "Micah Hamady", "Dan Knights",\
 "Antonio Gonzalez Pena"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Jesse Stombaugh"
 __email__ = "jesse.stombaugh@colorado.edu"
 __status__ = "Release"
@@ -793,9 +793,11 @@ def generate_3d_plots(prefs, data, custom_axes, background_color, label_color, \
     f = open(kinpath, 'w')
     f.write('\n'.join(res))
     f.close()
-    
+    basename,extension=os.path.splitext(filename)
+    filename='%s_3D_PCoA_plots' % (basename)
+ 
     #Write html page with the kinemage embedded
-    f2 = open(os.path.join(htmlpath,filename)+'_3D.html', 'w')
+    f2 = open(os.path.join(htmlpath,filename)+'.html', 'w')
     f2.write("<html><head></head><body><applet code='king/Kinglet.class' \
 archive='./jar/king.jar' width=800 height=600> \
 <param name='kinSource' value='%s'></body></html>" % (kinlink)) 
