@@ -2,11 +2,11 @@
 #file parse.py: parsers for map file, distance matrix file, env file
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2010, The QIIME Project"
+__copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Rob Knight", "Daniel McDonald", "Greg Caporaso",
     "Justin Kuczynski", "Cathy Lozupone", "Jens Reeder", "Antonio Gonzalez Pena"]
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -25,7 +25,7 @@ from cogent.core.tree import PhyloNode
 from copy import deepcopy
 import os
 import re
-from cogent.util.misc import revComp
+from cogent import DNA
 
 
 class QiimeParseError(Exception):
@@ -450,11 +450,23 @@ def parse_otu_table(lines,count_map_f=int):
                     if has_metadata:
                         # if there is OTU metadata the last column gets appended
                         # to the metadata list
-                        otu_table.append(array(map(count_map_f, fields[1:-1])))
+                        # added in a try/except to handle OTU tables containing
+                        # floating numbers
+                        try:
+                            otu_table.append(array(map(count_map_f,
+                                                       fields[1:-1])))
+                        except ValueError:
+                            otu_table.append(array(map(float, fields[1:-1])))
+                            
                         metadata.append(map(strip, fields[-1].split(';')))
                     else:
                         # otherwise all columns are appended to otu_table
-                        otu_table.append(array(map(count_map_f, fields[1:])))
+                        # added in a try/except to handle OTU tables containing
+                        # floating numbers
+                        try:
+                            otu_table.append(array(map(count_map_f,fields[1:])))
+                        except ValueError:
+                            otu_table.append(array(map(float, fields[1:])))
                         
     return sample_ids, otu_ids, array(otu_table), metadata
 
@@ -531,9 +543,7 @@ def parse_qiime_parameters(lines):
     """
     # The qiime_config object is a default dict: if keys are not
     # present, {} is returned
-    def return_empty_dict():
-        return dict()
-    result = defaultdict(return_empty_dict)
+    result = defaultdict(dict)
     
     for line in lines:
         line = line.strip()
@@ -552,10 +562,7 @@ def parse_qiime_parameters(lines):
             else:
                 pass
             
-            try:
-                result[script_id][parameter_id] = value
-            except KeyError:
-                result[script_id] = {parameter_id:value}
+            result[script_id][parameter_id] = value
     return result
 
 def sample_mapping_to_otu_table(lines):
@@ -696,7 +703,7 @@ def parse_illumina_line(l,barcode_length,rev_comp_barcode,
         barcode = y_position_subfields[1][:barcode_length]
     
     if rev_comp_barcode:
-        barcode = revComp(barcode)
+        barcode = DNA.rc(barcode)
     
     result = {\
      'Full description':':'.join(fields[:5]),\
@@ -714,7 +721,11 @@ def parse_illumina_line(l,barcode_length,rev_comp_barcode,
 
 def parse_qual_score(infile,value_cast_f=int):
     """Load quality scores into dict."""
-    id_to_qual = {}
+    id_to_qual = dict([rec for rec in MinimalQualParser(infile, value_cast_f)])
+    return id_to_qual
+        
+def MinimalQualParser(infile,value_cast_f=int):
+    """Yield quality scores"""
     for rec in FastaFinder(infile):
         curr_id = rec[0][1:]
         curr_qual = ' '.join(rec[1:])
@@ -723,8 +734,7 @@ def parse_qual_score(infile,value_cast_f=int):
         except ValueError:
             raise QiimeParseError,"Invalid qual file. Check the format of the qual files." 
         curr_pid = curr_id.split()[0]
-        id_to_qual[curr_pid] = parts
-    return id_to_qual
+        yield (curr_pid, parts)
 
 def parse_qual_scores(qual_files):
     """Load qual scores into dict of {id:qual_scores}.
@@ -798,3 +808,19 @@ def parse_trflp(lines):
         data.append(current_row)
     
     return sample_ids, otu_ids, array(data).transpose()
+    
+def parse_denoiser_mapping(denoiser_map):
+    """ read a denoiser mapping file into a dictionary """
+    result = {}
+    for line in denoiser_map:
+        line = line.strip().split('\t')
+        denoised_id = line[0].rstrip(':')
+        original_ids = [denoised_id] + line[1:]
+        if denoised_id in result:
+            # just a healthy dose of paranoia
+            raise ValueError, \
+             ("Duplicated identifiers in denoiser mapping file: "
+              "are you sure you merged the correct files?")
+        else:
+            result[denoised_id] = original_ids
+    return result
