@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 
 __author__ = "Greg Caporaso"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Rob Knight","Greg Caporaso", "Kyle Bittinger","Jens Reeder","William Walters"]
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -22,7 +22,6 @@ from os import makedirs
 from itertools import imap
 from cogent.parse.fasta import MinimalFastaParser
 from cogent.parse.mothur import parse_otu_list as mothur_parse
-from cogent.app.util import get_tmp_filename
 from cogent.app.cd_hit import cdhit_clusters_from_seqs
 from cogent.app.dotur import dotur_from_alignment
 from cogent.app.mothur import Mothur
@@ -33,7 +32,7 @@ from cogent.util.misc import remove_files
 from cogent import LoadSeqs, DNA, Alignment
 from cogent.util.trie import build_prefix_map
 from cogent.util.misc import flatten
-from qiime.util import FunctionWithParams
+from qiime.util import FunctionWithParams, get_tmp_filename
 from qiime.sort import sort_fasta_by_abundance
 from qiime.parse import fields_to_dict
 from qiime.pycogent_backports.uclust import get_clusters_from_fasta_filepath
@@ -105,6 +104,29 @@ class OtuPicker(FunctionWithParams):
             seq_id_map[data[1]] = data[0]
 
         return filtered_seqs, seq_id_map
+        
+    def _prefilter_exact_matches(self,seqs):
+        """
+        """
+        unique_sequences = {}
+        seq_id_map = {}
+        filtered_seqs = []
+        for seq_id,seq in seqs:
+            seq_id = seq_id.split()[0]
+            try:
+                temp_seq_id = unique_sequences[seq]
+            except KeyError:
+                # unseen sequence so create a new temp_seq_id,
+                # a new unique_sequence entry, and new seq_id_map 
+                # entry, and add the sequence to the list of 
+                # filtered seqs -- this will retain the order
+                # of the input sequences too
+                temp_seq_id = 'QiimeExactMatch.%s' % seq_id
+                unique_sequences[seq] = temp_seq_id
+                seq_id_map[temp_seq_id] = []
+                filtered_seqs.append((temp_seq_id,seq))
+            seq_id_map[temp_seq_id].append(seq_id)
+        return filtered_seqs, seq_id_map
 
 
     def _prefilter_with_trie(self, seq_path):
@@ -680,7 +702,6 @@ class UclustOtuPickerBase(OtuPicker):
         # Get a temp file name for the sorted fasta file
         sorted_input_seqs_filepath = \
          get_tmp_filename(prefix=self.Name,suffix='.fasta')
-        
         # Sort input seqs by abundance, and write to the temp
         # file
         sort_fasta_by_abundance(open(seq_path,'U'),
@@ -716,6 +737,22 @@ class UclustOtuPickerBase(OtuPicker):
             log_lines.append('Result path: None, returned as dict.')
             
         return result
+    
+    def _apply_identical_sequences_prefilter(self,seq_path):
+        """ """
+        unique_seqs_fp = get_tmp_filename(
+         prefix='UclustExactMatchFilter',suffix='.fasta')
+        seqs_to_cluster, exact_match_id_map =\
+         self._prefilter_exact_matches(MinimalFastaParser(open(seq_path,'U')))
+        self.files_to_remove.append(unique_seqs_fp)
+        unique_seqs_f = open(unique_seqs_fp,'w')
+        for seq_id,seq in seqs_to_cluster:
+            unique_seqs_f.write('>%s\n%s\n' % (seq_id,seq))
+        unique_seqs_f.close()
+        # clean up the seqs_to_cluster list as it can be big and we
+        # don't need it again
+        # del(seqs_to_cluster)
+        return exact_match_id_map, unique_seqs_fp
 
 class UclustOtuPicker(UclustOtuPickerBase):
     """ Uclust based OTU picker
@@ -753,7 +790,8 @@ class UclustOtuPicker(UclustOtuPickerBase):
          'new_cluster_identifier':None,
          'stable_sort':True,
          'save_uc_files':True,
-         'output_dir':'.'}
+         'output_dir':'.',
+         'prefilter_identical_sequences':True}
         _params.update(params)
         OtuPicker.__init__(self, _params)
     
@@ -771,18 +809,21 @@ class UclustOtuPicker(UclustOtuPickerBase):
         log_path: path to log, which includes dump of params.
 
         """
-        
+        prefilter_identical_sequences =\
+         self.Params['prefilter_identical_sequences']
         original_fasta_path = seq_path
+        self.files_to_remove = []
         
         if self.Params['presort_by_abundance']:
             # seq path will become the temporary sorted sequences
             # filepath, to be cleaned up after the run
             seq_path = self._presort_by_abundance(seq_path)
-            files_to_remove = [seq_path]
-        else:
-            # create a dummy list of files to clean up
-            files_to_remove = []
+            self.files_to_remove.append(seq_path)
         
+        # Collapse idetical sequences to a new file
+        if prefilter_identical_sequences:
+            exact_match_id_map, seq_path =\
+             self._apply_identical_sequences_prefilter(seq_path)
         
         # perform the clustering
         clusters, failures, seeds = get_clusters_from_fasta_filepath(
@@ -804,11 +845,16 @@ class UclustOtuPicker(UclustOtuPickerBase):
          HALT_EXEC=HALT_EXEC)
         
         # clean up any temp files that were created
-        remove_files(files_to_remove)
+        remove_files(self.files_to_remove)
         
         log_lines = []
         log_lines.append('Num OTUs:%d' % len(clusters))
         
+        # expand identical sequences to create full OTU map
+        if prefilter_identical_sequences:
+            clusters = self._map_filtered_clusters_to_full_clusters(
+                        clusters,exact_match_id_map)
+        
         otu_id_prefix = self.Params['new_cluster_identifier']
         if otu_id_prefix == None:
             clusters = enumerate(clusters)
@@ -850,7 +896,8 @@ class UclustReferenceOtuPicker(UclustOtuPickerBase):
                    'presort_by_abundance':True,
                    'stable_sort':True,
                    'save_uc_files':True,
-                   'output_dir':'.'}
+                   'output_dir':'.',
+                   'prefilter_identical_sequences':True}
         _params.update(params)
         OtuPicker.__init__(self, _params)
     
@@ -865,21 +912,25 @@ class UclustReferenceOtuPicker(UclustOtuPickerBase):
                  HALT_EXEC=False):
                      
         original_fasta_path = seq_fp
+        prefilter_identical_sequences =\
+         self.Params['prefilter_identical_sequences']
         
         if new_cluster_identifier:
             self.Params['new_cluster_identifier'] = new_cluster_identifier
         if next_new_cluster_number != None:
             self.Params['next_new_cluster_number'] = next_new_cluster_number
-            
+        self.files_to_remove = []
+        
         if self.Params['presort_by_abundance']:
             # seq path will become the temporary sorted sequences
             # filepath, to be cleaned up after the run
             seq_fp = self._presort_by_abundance(seq_fp)
-            files_to_remove = [seq_fp]
-        else:
-            # create a dummy list of files to clean up
-            files_to_remove = []
+            self.files_to_remove.append(seq_fp)
 
+        # Collapse idetical sequences to a new file
+        if prefilter_identical_sequences:
+            exact_match_id_map, seq_fp =\
+             self._apply_identical_sequences_prefilter(seq_fp)
         
         # perform the clustering
         cluster_map, failures, new_seeds = get_clusters_from_fasta_filepath(
@@ -901,13 +952,28 @@ class UclustReferenceOtuPicker(UclustOtuPickerBase):
             save_uc_files=self.Params['save_uc_files'],
             output_dir=self.Params['output_dir'],
             HALT_EXEC=HALT_EXEC)
-            
         
+        # expand identical sequences to create full OTU map
+        if prefilter_identical_sequences:
+            # expand the clusters (while retaining the names of
+            # the clusters so we know which are new OTUs and
+            # which are reference OTUs)
+            cluster_names = cluster_map.keys()
+            clusters = [cluster_map[c] for c in cluster_names]
+            clusters = self._map_filtered_clusters_to_full_clusters(
+                        clusters,exact_match_id_map)
+            cluster_map = dict(zip(cluster_names,clusters))
+            
+            # expand failures
+            temp_failures = []
+            for fa in failures:
+                temp_failures.extend(exact_match_id_map[fa])
+            failures = temp_failures
         
         self._rename_clusters(cluster_map,new_seeds)
         
         # clean up any temp files that were created
-        remove_files(files_to_remove)
+        remove_files(self.files_to_remove)
         
         log_lines = []
         log_lines.append('Reference seqs:%s' % refseqs_fp)
