@@ -11,7 +11,7 @@ Modified from cogent.app.cd_hit.py on 1-21-10, written by Daniel McDonald.
 """
 
 __author__ = "William Walters"
-__copyright__ = "Copyright 2007-2009, The Cogent Project"
+__copyright__ = "Copyright 2007-2011, The PyCogent Project"
 __credits__ = ["William Walters","Greg Caporaso"]
 __license__ = "GPL"
 __version__ = "1.2.0.dev"
@@ -25,7 +25,8 @@ from cogent.parse.fasta import MinimalFastaParser
 from cogent.app.parameters import ValuedParameter, FlagParameter
 from cogent.app.util import CommandLineApplication, ResultPath,\
  get_tmp_filename, ApplicationError, ApplicationNotFoundError
-from cogent.util.misc import revComp as reverse_complement, remove_files
+from cogent.util.misc import remove_files
+from cogent import DNA
 
 class UclustParseError(Exception):
     pass
@@ -237,9 +238,9 @@ def process_uclust_pw_alignment_results(fasta_pairs_lines,uc_lines):
             
         if target_rev_match:
             query_id = uc_query_id + ' RC'
-            aligned_query = reverse_complement(aligned_query)
+            aligned_query = DNA.rc(aligned_query)
             target_id = uc_target_id
-            aligned_target = reverse_complement(aligned_target)
+            aligned_target = DNA.rc(aligned_target)
         else:
             query_id = uc_query_id
             aligned_query = aligned_query
@@ -545,7 +546,7 @@ def get_clusters_from_fasta_filepath(
     except ApplicationError:
         remove_files(files_to_remove)
         raise ApplicationError, ('Error running uclust. Possible causes are '
-         'unsupported version (current supported version is v1.2.16) is installed or '
+         'unsupported version (current supported version is v1.2.22) is installed or '
          'improperly formatted input file was provided')
     except ApplicationNotFoundError:
         remove_files(files_to_remove)
