@@ -25,10 +25,10 @@ have a barcode that matches the mapping file will not be recorded.
 """
 
 __author__ = "Rob Knight and Micah Hamady"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Rob Knight", "Micah Hamady", "Greg Caporaso", "Kyle Bittinger","Jesse Stombaugh","William Walters","Jens Reeder"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "William Walters"
 __email__ = "rob@spot.colorado.edu, william.a.walters@colorado.edu"
 __status__ = "Release"
@@ -48,6 +48,7 @@ from cogent.seqsim.sequence_generators import SequenceGenerator, IUPAC_DNA
 from cogent.core.moltype import IUPAC_DNA_ambiguities
 from cogent import DNA, LoadSeqs
 from cogent.align.align import make_dna_scoring_dict, local_pairwise
+from cogent.util.misc import remove_files 
 
 from qiime.check_id_map import process_id_map
 from qiime.barcode import correct_barcode
@@ -55,7 +56,7 @@ from qiime.hamming import decode_barcode_8
 from qiime.golay import decode as decode_golay_12
 from qiime.format import format_histograms
 from qiime.parse import QiimeParseError, parse_qual_scores
-from qiime.util import create_dir
+from qiime.util import create_dir, median_absolute_deviation
 
 ## Including new=True in the histogram() call is necessary to 
 ## get the correct result in versions prior to NumPy 1.2.0,
@@ -261,20 +262,36 @@ def expand_degeneracies(raw_primers):
     return expanded_primers
     
 
-def check_map(infile, has_barcodes=True, disable_primer_check=False):
+def check_map(infile, disable_primer_check, added_demultiplex_field=None,
+              has_barcodes=True):
     """Check mapping file and extract list of valid barcodes, primers """
+    
+
+    
     hds, id_map, dsp, run_description, errors, warnings = \
         process_id_map(infile, is_barcoded=has_barcodes, \
-        disable_primer_check=disable_primer_check)
+        disable_primer_check=disable_primer_check,
+        added_demultiplex_field=added_demultiplex_field)
+        
+    
     barcode_to_sample_id = {}
     
     primer_seqs_lens = {}
     all_primers = {}
 
+
     for sample_id, sample in id_map.items():
-        barcode_to_sample_id[sample['BarcodeSequence'].upper()] = sample_id
+        if added_demultiplex_field:
+            barcode_to_sample_id[sample['BarcodeSequence'].upper() + "," +\
+             sample[added_demultiplex_field]] = sample_id
+        else:
+            barcode_to_sample_id[sample['BarcodeSequence'].upper()] = sample_id
         if not disable_primer_check:
             raw_primers = sample['LinkerPrimerSequence'].upper().split(',')
+            
+            if len(raw_primers[0].strip()) == 0:
+                raise ValueError,('No primers detected, please use the '+\
+                 '-p parameter to disable primer detection.')
             expanded_primers = expand_degeneracies(raw_primers)
             curr_bc_primers = {}
             for primer in expanded_primers:
@@ -282,7 +299,7 @@ def check_map(infile, has_barcodes=True, disable_primer_check=False):
                 all_primers[primer] = len(primer)
             primer_seqs_lens[sample['BarcodeSequence']] = curr_bc_primers
     
-    
+
     
     
     return hds, id_map, barcode_to_sample_id, warnings, errors, \
@@ -341,25 +358,85 @@ def seq_exceeds_homopolymers(curr_seq, max_len=6):
     return False
 
 def check_barcode(curr_barcode, barcode_type, valid_map,
- attempt_correction=True):
+                  attempt_correction=True, added_demultiplex_field=None,
+                  curr_id=None):
     """Return whether barcode is valid, and attempt correction."""
     
+    
     corrected_bc = False
-    if curr_barcode in valid_map:
-        return False, curr_barcode, corrected_bc
-    elif attempt_correction == False:
-        return True, curr_barcode, corrected_bc
+    
+
+    if added_demultiplex_field:
+        
+        added_demultiplex_lens =\
+         set([len(bc.split(',')[1]) for bc in valid_map])
+         
+        # using set() will put in order of smallest to largest and removes
+        # redundant lengths, converting to list to sort from largest to smallest
+        added_demultiplex_lens =\
+         [length for length in added_demultiplex_lens][::-1]
+             
+
+        
+        # Handle specific case of run_prefix
+        # Need to slice out size(s) of label that matches run prefix size(s)
+        if added_demultiplex_field.upper() == "RUN_PREFIX":
+            added_demultiplex =\
+             [curr_id.split()[0][0:added_demultiplex_len] for \
+             added_demultiplex_len in added_demultiplex_lens]
+
+        else:
+            for label_item in curr_id.split():
+                if label_item.startswith(added_demultiplex_field):
+                    added_demultiplex = [label_item.split('=')[1]]
+                    
+        all_bcs = [bc.split(',')[0] for bc in valid_map]
+        all_added_demultiplex = [bc.split(',')[1] for bc in valid_map]
+
+        
+        for curr_added_demultiplex in added_demultiplex:
+
+            bc_and_demultiplex = curr_barcode + "," + curr_added_demultiplex
+            
+
+            
+            if bc_and_demultiplex in valid_map:
+                return False, bc_and_demultiplex, corrected_bc
+            elif attempt_correction == False:
+                return True, curr_barcode, corrected_bc
+    else:
+        if curr_barcode in valid_map:
+            return False, curr_barcode, corrected_bc
+        elif attempt_correction == False:
+            return True, curr_barcode, corrected_bc
     
     if barcode_type in BARCODE_TYPES:
         expect_len, curr_bc_fun  = BARCODE_TYPES[barcode_type]
         barcode, num_errors = curr_bc_fun(curr_barcode, valid_map)
         corrected_bc = True
-        return num_errors, barcode, corrected_bc
+
+        if added_demultiplex_field:
+
+            for curr_added_demultiplex in added_demultiplex:
+                bc_and_demultiplex = barcode + "," + curr_added_demultiplex
+                if bc_and_demultiplex in valid_map:
+                    return num_errors, bc_and_demultiplex, corrected_bc
+
+        else:
+            return num_errors, barcode, corrected_bc
     else:
         try:
             expect_len, curr_bc_fun = int(barcode_type), correct_barcode
             barcode, num_errors = curr_bc_fun(curr_barcode, valid_map)
             corrected_bc = True
+            
+            if added_demultiplex_field:
+
+                for curr_added_demultiplex in added_demultiplex:
+                    bc_and_demultiplex = barcode + "," + curr_added_demultiplex
+                    if bc_and_demultiplex in valid_map:
+                        return num_errors, bc_and_demultiplex, corrected_bc
+                    
         except ValueError:
             raise ValueError, "Unsupported barcode type: %s" % barcode_type
         return num_errors, barcode, corrected_bc
@@ -449,20 +526,25 @@ def check_window_qual_scores(qual_scores, window=50, min_average=25):
 
 def check_seqs(fasta_out, fasta_files, starting_ix, valid_map, qual_mappings, 
     filters, barcode_len, keep_primer, keep_barcode, barcode_type, 
-    max_bc_errors,remove_unassigned, attempt_bc_correction,
+    max_bc_errors, retain_unassigned_reads, attempt_bc_correction,
     primer_seqs_lens, all_primers, max_primer_mm, disable_primer_check,
     reverse_primers, rev_primers, qual_out, qual_score_window=0,
-    discard_bad_windows=False, min_qual_score=25, min_seq_len=200):
+    discard_bad_windows=False, min_qual_score=25, min_seq_len=200,
+    median_length_filtering=None, added_demultiplex_field=None):
     """Checks fasta-format sequences and qual files for validity."""
+    
+    
     seq_lengths = {}
     bc_counts = defaultdict(list)
     curr_ix = starting_ix
     corr_ct = 0 #count of corrected barcodes
 
     # get the list of barcode lengths in reverse order
-    barcode_length_order = list(set([len(bc) for bc in valid_map]))
+    barcode_length_order =\
+     list(set([len(bc.split(',')[0]) for bc in valid_map]))
     barcode_length_order.sort()
     barcode_length_order = barcode_length_order[::-1]
+    
 
     primer_mismatch_count = 0
     all_primers_lens = list(set(all_primers.values()))
@@ -478,6 +560,7 @@ def check_seqs(fasta_out, fasta_files, starting_ix, valid_map, qual_mappings,
     for fasta_in in fasta_files:
         for curr_id, curr_seq in MinimalFastaParser(fasta_in):
             curr_rid = curr_id.split()[0]
+
             curr_len = len(curr_seq)
             curr_qual = qual_mappings.get(curr_rid, None)
             
@@ -492,16 +575,16 @@ def check_seqs(fasta_out, fasta_files, starting_ix, valid_map, qual_mappings,
             if failed:  #if we failed any of the checks, bail out here
                 bc_counts['#FAILED'].append(curr_rid)
                 continue
-                
             
-
-                
                 
             if barcode_type == 'variable_length':
                 # Reset the raw_barcode, raw_seq, and barcode_len -- if 
                 # we don't match a barcode from the mapping file, we want
                 # these values to be None
                 raw_barcode, raw_seq, barcode_len = (None, None, None)
+                
+                curr_valid_map =\
+                 [curr_bc.split(',')[0] for curr_bc in valid_map]
                 # Iterate through the barcode length from longest to shortest
                 for l in barcode_length_order:
                     # extract the current length barcode from the sequence
@@ -509,7 +592,7 @@ def check_seqs(fasta_out, fasta_files, starting_ix, valid_map, qual_mappings,
                     # check if the sliced sequence corresponds to a valid
                     # barcode, and if so set raw_barcode, raw_seq, and 
                     # barcode_len for use in the next steps
-                    if bc in valid_map:
+                    if bc in curr_valid_map:
                         raw_barcode, raw_seq = bc, seq
                         barcode_len = len(raw_barcode)
                         break
@@ -571,7 +654,7 @@ def check_seqs(fasta_out, fasta_files, starting_ix, valid_map, qual_mappings,
             try:
                 bc_diffs, curr_bc, corrected_bc = \
                     check_barcode(cbc, barcode_type, valid_map.keys(), \
-                    attempt_bc_correction)
+                    attempt_bc_correction, added_demultiplex_field, curr_id)
                 if bc_diffs > max_bc_errors:
                     raise ValueError, "Too many errors in barcode"
                 corr_ct += bool(corrected_bc)
@@ -638,9 +721,6 @@ def check_seqs(fasta_out, fasta_files, starting_ix, valid_map, qual_mappings,
                         below_seq_min_after_trunc += 1
                         
                     
-                 
-
-                    
             # Slice out regions of quality scores that correspond to the 
             # written sequence, i.e., remove the barcodes/primers and reverse
             # primers if option is enabled.
@@ -672,28 +752,122 @@ def check_seqs(fasta_out, fasta_files, starting_ix, valid_map, qual_mappings,
             
             # Record number of seqs associated with particular barcode.
             bc_counts[curr_bc].append(curr_rid)
-                 
-            if remove_unassigned or (not attempt_bc_correction):
-                if curr_samp_id!="Unassigned":
-                    fasta_out.write(">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s\n" % 
-                     (new_id, curr_rid, cbc, curr_bc, int(bc_diffs), write_seq))
-                    if qual_out:
-                        qual_out.write(">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s" % 
-                         (new_id, curr_rid, cbc, curr_bc, int(bc_diffs), qual_scores_out))
-                else:
-                    bc_counts['#FAILED'].append(curr_rid)
-            else:    
-                fasta_out.write(">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s\n" % 
-                    (new_id, curr_rid, cbc, curr_bc, int(bc_diffs), write_seq))
+            
+            if retain_unassigned_reads and curr_samp_id == "Unassigned":
+                fasta_out.write(
+                 ">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s\n" % 
+                 (new_id, curr_rid, cbc, curr_bc, int(bc_diffs), write_seq))
                 if qual_out:
-                        qual_out.write(">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s" % 
-                         (new_id, curr_rid, cbc, curr_bc, int(bc_diffs), qual_scores_out))
+                    qual_out.write(
+                     ">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s" % 
+                     (new_id, curr_rid, cbc, curr_bc, int(bc_diffs),
+                     qual_scores_out))
+            elif not retain_unassigned_reads and curr_samp_id == "Unassigned":
+                 bc_counts['#FAILED'].append(curr_rid)
+            else:
+                fasta_out.write(
+                 ">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s\n" % 
+                 (new_id, curr_rid, cbc, curr_bc, int(bc_diffs), write_seq))
+                if qual_out:
+                    qual_out.write(
+                     ">%s %s orig_bc=%s new_bc=%s bc_diffs=%s\n%s" % 
+                     (new_id, curr_rid, cbc, curr_bc, int(bc_diffs),
+                     qual_scores_out))
+                 
+                         
+            curr_len = len(curr_seq)
+
+            seq_lengths[curr_rid] = curr_len
+            
             curr_ix += 1
+            
+    if median_length_filtering:
+        # Read original fasta file output to get sequence lengths
+        fasta_out.close()
+        fasta_out = open(fasta_out.name, "U")
+        
+        # Record sequence lengths for median/mad calculation
+        sequence_lens = []
+        for label, seq in MinimalFastaParser(fasta_out):
+            sequence_lens.append(len(seq))
+        
+        
+        
+        '''# Create a temporary file to copy the contents of the fasta file, will
+        # need to delete once operations complete.
+        fasta_temp = open(fasta_out.name + "_tmp.fasta", "w")
+        
+        sequence_lens = []
+        for label, seq in MinimalFastaParser(fasta_lens):
+            sequence_lens.append(len(seq))
+            fasta_temp.write(">%s\n%s\n" % (label, seq))
+            
+        fasta_temp.close()
+        fasta_temp = open(fasta_out.name + "_tmp.fasta", "U")
+        
+        fasta_lens.close()
+        # Overwrite seqs.fna with length filtered data
+        fasta_out = open(fasta_out.name, "w")'''
+        
+        med_abs_dev, med_length = median_absolute_deviation(sequence_lens)
+        
+        min_corrected_len = med_length - med_abs_dev *\
+         float(median_length_filtering)
+        max_corrected_len = med_length + med_abs_dev *\
+         float(median_length_filtering)
+        seqs_discarded_median = 0
+
+        fasta_out.seek(0)
+        
+        final_written_lens = []
+        
+        # Create final seqs.fna
+        final_fasta_out = open(fasta_out.name.replace('.tmp',''), "w")
+        
+        for label,seq in MinimalFastaParser(fasta_out):
+            curr_len = len(seq)
+            if curr_len < min_corrected_len or curr_len > max_corrected_len:
+                seqs_discarded_median += 1
+            else:
+                final_fasta_out.write(">%s\n%s\n" % (label,seq))
+                final_written_lens.append(len(seq))
+        
+        final_fasta_out.close()
+        fasta_out.close()
+        remove_files([fasta_out.name])
+        
+    else:
+        min_corrected_len = 0
+        max_corrected_len = 0
+        seqs_discarded_median = 0
+        final_written_lens = 0
+        
+        # Copy tmp seqs file to final seqs.fna file
+        fasta_out.close()
+        fasta_out = open(fasta_out.name, "U")
+        
+        # Create final seqs.fna
+        final_fasta_out = open(fasta_out.name.replace('.tmp',''), "w")
+        
+        for label,seq in MinimalFastaParser(fasta_out):
+                final_fasta_out.write(">%s\n%s\n" % (label,seq))
+        
+        final_fasta_out.close()
+        fasta_out.close()
+        remove_files([fasta_out.name])
+        
+    median_results = (median_length_filtering, min_corrected_len, 
+     max_corrected_len, seqs_discarded_median, final_written_lens)
+        
+
+
+            
+    
     log_out = format_log(bc_counts, corr_ct, seq_lengths, valid_map, filters,\
-     remove_unassigned, attempt_bc_correction, primer_mismatch_count, \
+     retain_unassigned_reads, attempt_bc_correction, primer_mismatch_count, \
      max_primer_mm, reverse_primers, reverse_primer_not_found,
      sliding_window_failed, below_seq_min_after_trunc, qual_score_window, 
-     discard_bad_windows, min_seq_len)
+     discard_bad_windows, min_seq_len, median_results)
     all_seq_lengths, good_seq_lengths = get_seq_lengths(seq_lengths, bc_counts)
     return log_out, all_seq_lengths, good_seq_lengths
 
@@ -719,9 +893,10 @@ def format_qual_output(qual_array):
     
 
 def format_log(bc_counts, corr_ct, seq_lengths, valid_map, filters,\
-remove_unassigned, attempt_bc_correction, primer_mismatch_count, max_primer_mm,\
+retain_unassigned_reads, attempt_bc_correction, primer_mismatch_count, max_primer_mm,\
 reverse_primers, reverse_primer_not_found, sliding_window_failed,
-below_seq_min_after_trunc, qual_score_window, discard_bad_windows, min_seq_len):
+below_seq_min_after_trunc, qual_score_window, discard_bad_windows, min_seq_len,
+median_results=(None)):
     """Makes log lines"""
 
     
@@ -729,6 +904,15 @@ below_seq_min_after_trunc, qual_score_window, discard_bad_windows, min_seq_len):
     all_seq_lengths, good_seq_lengths = get_seq_lengths(seq_lengths, bc_counts)
     log_out.append("Number raw input seqs\t%d\n" % len(seq_lengths)) 
     
+    # append log data for median absolute deviation sequence length filtering
+    # if was performed.
+    if median_results[0]:
+        log_out.append("Specified allowed number of median absolute "+\
+         "deviations for sequence retention: %3.2f" % (float(median_results[0])))
+        log_out.append("Sequences with lengths outside bounds of "+\
+         "%d and %d: %d\n" %\
+          (median_results[1], median_results[2], median_results[3]))
+    
     for f in filters:
         log_out.append(str(f))
     log_out.append('Num mismatches in primer exceeds limit of %s: %d\n' %\
@@ -765,9 +949,16 @@ below_seq_min_after_trunc, qual_score_window, discard_bad_windows, min_seq_len):
     log_out.append("Raw len min/max/avg\t%.1f/%.1f/%.1f" % 
         (min(all_seq_lengths), max(all_seq_lengths), mean(all_seq_lengths)))
     
-    if good_seq_lengths:
+    if median_results[0]:
         log_out.append("Wrote len min/max/avg\t%.1f/%.1f/%.1f" % 
-        (min(good_seq_lengths), max(good_seq_lengths), mean(good_seq_lengths))) 
+            (min(median_results[4]), max(median_results[4]),
+            mean(median_results[4])))
+    else:
+        if good_seq_lengths:
+            log_out.append("Wrote len min/max/avg\t%.1f/%.1f/%.1f" % 
+            (min(good_seq_lengths), max(good_seq_lengths),
+            mean(good_seq_lengths)))
+
     
     #figure out which barcodes we got that didn't come from valid samples
     valid_bc_nomap = set(bc_counts) - set(valid_map) - set([None,'#FAILED'])
@@ -780,8 +971,8 @@ below_seq_min_after_trunc, qual_score_window, discard_bad_windows, min_seq_len):
         log_out.append("Uncorrected barcodes will not be written to the "+\
         "output fasta file.\nCorrected barcodes will be written with "+\
         "the appropriate barcode category.\nCorrected but unassigned "+\
-        "sequences will be written as such unless disabled via the -r "+\
-        "option.\n")
+        "sequences will not be written unless --retain_unassigned_reads "+\
+        "is enabled.\n")
     else:
         log_out.append("Barcode correction has been disabled via the -c "+\
         "option.\n")
@@ -791,16 +982,17 @@ below_seq_min_after_trunc, qual_score_window, discard_bad_windows, min_seq_len):
         log_out.append("Barcodes not in mapping file\tCount")
         for count, bc in reversed(sorted(valid_bc_nomap_counts)):
             log_out.append("%s\t%d" % (bc, count))
-    if remove_unassigned:
+    if retain_unassigned_reads:
         log_out.append("Sequences associated with valid barcodes that are not"+\
-        " in the mapping file will not be written. -r option enabled.")
+        " in the mapping file will be written. --retain_unassigned_reads "+\
+        "option enabled.")
     elif not attempt_bc_correction:
         log_out.append("Barcode correction has been disabled (-c option), "+\
         "no unassigned or invalid barcode sequences will be recorded.")
     else:
         log_out.append("Sequences associated with valid barcodes that are "+\
-        "not in the mapping file will be written as 'unassigned'.  -r option "+\
-        "disabled.")
+        "not in the mapping file will not be written.")
+        
     log_out.append("\nBarcodes in mapping file")
    
     sample_cts = [(len(bc_counts[bc]), bc, sample_id) for bc, sample_id 
@@ -817,8 +1009,12 @@ below_seq_min_after_trunc, qual_score_window, discard_bad_windows, min_seq_len):
             log_out.append("%s\t%d\t%s" % (sample_id, count, bc))
 
         
-        
-    log_out.append("\nTotal number seqs written\t%d" % len(good_seq_lengths)) 
+    if median_results[0]:
+        corrected_written_seqs = len(good_seq_lengths) - median_results[3]
+    else:
+        corrected_written_seqs = len(good_seq_lengths)
+    
+    log_out.append("\nTotal number seqs written\t%d" % corrected_written_seqs) 
     return log_out
     
 def get_reverse_primers(id_map):
@@ -836,13 +1032,14 @@ def get_reverse_primers(id_map):
 def preprocess(fasta_files, qual_files, mapping_file, 
     barcode_type="golay_12",
     min_seq_len=200, max_seq_len=1000, min_qual_score=25, starting_ix=1,
-    keep_primer=True, max_ambig=0, max_primer_mm=1, trim_seq_len=True,
-    dir_prefix='.', max_bc_errors=2, max_homopolymer=4,remove_unassigned=False,
-    keep_barcode=False, attempt_bc_correction=True, qual_score_window=0,
-    disable_primers=False, reverse_primers='disable', record_qual_scores=False,
-    discard_bad_windows=False):
+    keep_primer=False, max_ambig=0, max_primer_mm=0, trim_seq_len=False,
+    dir_prefix='.', max_bc_errors=2, max_homopolymer=4,
+    retain_unassigned_reads=False, keep_barcode=False, 
+    attempt_bc_correction=True, qual_score_window=0,
+    disable_primer_check=False, reverse_primers='disable', 
+    record_qual_scores=False, discard_bad_windows=False, 
+    median_length_filtering=None, added_demultiplex_field=None):
         
-
         
     """
     Preprocess barcoded libraries, e.g. from 454.
@@ -875,7 +1072,7 @@ def preprocess(fasta_files, qual_files, mapping_file,
 
     max_primer_mm: maximum number of primer mismatches to allow.
 
-    trim_seq_len: if True (default), calculates lengths after trimming.
+    trim_seq_len: if True, calculates lengths after trimming.
 
     dir_prefix: prefix of directories to write files into.
 
@@ -884,14 +1081,14 @@ def preprocess(fasta_files, qual_files, mapping_file,
     max_homopolymer: maximum number of a nucleotide that can be 
     repeated in a given sequence.
     
-    remove_unassigned: If True (False default), will not write seqs to the
+    retain_unassigned_reads: If True (False default), will write seqs to the
     output .fna file that have a valid barcode (by Golay or Hamming standard)
-    but are not included in the input mapping file.
+    but are not included in the input mapping file, as "Unassigned."
     
     attempt_bc_correction: (default True) will attempt to find nearest valid
     barcode.  Can be disabled to improve performance.
     
-    disable_primers: (default False) Disables testing for primers in the
+    disable_primer_check: (default False) Disables testing for primers in the
     input mapping file and primer testing in the input sequence files.
     
     reverse_primers: (default 'disable') Enables removal of reverse primers and
@@ -916,6 +1113,22 @@ def preprocess(fasta_files, qual_files, mapping_file,
     discard_bad_windows: (default False) If True, will completely discard
     sequences that have a low quality window.  If False, sequences will be
     truncated to the first base of the bad window.
+    
+    median_length_filtering: (default None) If a value is specified, will 
+    disable all min and max length filtering, and instead will calculate the
+    median length of all sequences output, and instead filter out sequences
+    based upon whether or not they fall within the number of median absolute
+    deviations given by this parameter.
+    
+    added_demultiplex_field: (default None) If enabled, will attempt to 
+    demultiplex by both a barcode and data that can be parsed from the fasta
+    label/comment.  If 'run_prefix' is specified will pull the character string
+    immediatly following the '>' at the beginning of the fasta label.  Any 
+    other field specified will be pulled from the comment.  Example: if 'plate'
+    is specified, the string following 'plate=' will be used.  The mapping file
+    must contain a column with a header matching the name specified by the -j
+    option, and every combination of barcode + added demultiplex option must
+    be unique.
 
     Result:
     in dir_prefix, writes the following files:
@@ -938,8 +1151,6 @@ def preprocess(fasta_files, qual_files, mapping_file,
         raise ValueError, "Max ambig chars must be >= 0."
     if max_primer_mm < 0:
         raise ValueError, "Max primer mismatches must be >= 0."
-    if min_qual_score < 5:
-        raise ValueError, "Min qual score must be >= 5."
     if reverse_primers not in ['disable','truncate_only','truncate_remove']:
         raise ValueError, ("reverse_primers parameter must be 'disable', "+\
          "truncate_only, or truncate_remove.")
@@ -954,12 +1165,15 @@ def preprocess(fasta_files, qual_files, mapping_file,
     """# Generate primer sequence patterns - changing to mapping file primers.
     all_primer_seqs, primer_seq_len = \
         get_primer_seqs(primer_seq_pats.split(',')) """
-        
+    
+
     # Check mapping file and get barcode mapping 
     map_file = open(mapping_file, 'U')
     headers, id_map, valid_map, warnings, errors, \
-     primer_seqs_lens, all_primers = check_map(map_file, \
-     disable_primer_check = disable_primers )
+     primer_seqs_lens, all_primers = check_map(map_file, disable_primer_check,
+     added_demultiplex_field)
+     
+
      
     if reverse_primers != 'disable':
         if 'ReversePrimer' not in headers:
@@ -983,9 +1197,11 @@ def preprocess(fasta_files, qual_files, mapping_file,
         "Validate with check_id_map first: %s" % "\n".join(errors)
 
     # Find actual length of barcodes in the mapping file, also check for
-    # variable lengths
-    barcode_length_check = list(set([len(bc) for bc in valid_map]))
-    # Check barcode type
+    # variable lengths, in case of added_demultiplex, split on comma.
+    barcode_length_check =\
+     list(set([len(bc.split(',')[0]) for bc in valid_map]))
+    
+        # Check barcode type
     if barcode_type not in BARCODE_TYPES:
         try:
             barcode_len, barcode_fun = int(barcode_type), correct_barcode
@@ -999,6 +1215,8 @@ def preprocess(fasta_files, qual_files, mapping_file,
     # of the barcodes used, a check on the actual barcode lengths needs to
     # be done, and an exception raised if they are variable length and not
     # specified as so.
+
+    
     if barcode_type != "variable_length":
         # Raise error if variable length barcodes are present but not
         # specified
@@ -1045,25 +1263,31 @@ def preprocess(fasta_files, qual_files, mapping_file,
 
     #make filters
     filters = []
-    #seq len filter depends on whether we're including the barcode
-    if trim_seq_len:
-        # This processing occurs before primer testing, will use largest
-        # primer length to calculate lengths.  the dict all_primers has
-        # keys of each primer with the length of said primer as the value
-        if not disable_primer_check:
-            primer_seq_len = max(all_primers.values())
+    #seq len filter depends on whether we're including the barcode, if 
+    # median_length_filtering turned on, no length filtering.
+    if not median_length_filtering:
+        if trim_seq_len:
+            # This processing occurs before primer testing, will use largest
+            # primer length to calculate lengths.  the dict all_primers has
+            # keys of each primer with the length of said primer as the value
+            if disable_primer_check:
+                primer_seq_len = 0
+            else:
+                primer_seq_len = max(all_primers.values())
+                
+            if barcode_type == "variable_length":
+                barcode_len = max(barcode_length_check)
+            
+            trim = barcode_len + primer_seq_len
+            filters.append(SeqQualBad(
+                'Length outside bounds of %s and %s' % (min_seq_len,max_seq_len),
+                lambda id_, seq, qual: \
+                    not (min_seq_len<=len(seq)-trim<= max_seq_len)))
         else:
-            # Set to zero if primers not used
-            primer_seq_len = 0
-        trim = barcode_len + primer_seq_len
-        filters.append(SeqQualBad(
-            'Length outside bounds of %s and %s' % (min_seq_len,max_seq_len),
-            lambda id_, seq, qual: \
-                not (min_seq_len<=len(seq)-trim<= max_seq_len)))
-    else:
-        filters.append(SeqQualBad(
-            'Length outside bounds of %s and %s' % (min_seq_len,max_seq_len),
-            lambda id_, seq, qual: not (min_seq_len<=len(seq)<= max_seq_len)))
+            filters.append(SeqQualBad(
+                'Length outside bounds of %s and %s' % (min_seq_len,max_seq_len),
+                lambda id_, seq, qual: not (min_seq_len<=len(seq)<= max_seq_len)))
+    
     filters.append(SeqQualBad(
         'Num ambiguous bases exceeds limit of %s' % max_ambig,
         lambda id_, seq, qual: count_ambig(seq) > max_ambig))
@@ -1088,7 +1312,7 @@ def preprocess(fasta_files, qual_files, mapping_file,
             seq[barcode_len:], max_homopolymer)))
 
     # Check seqs and write out
-    fasta_out = open(dir_prefix + '/' + 'seqs.fna', 'w+')
+    fasta_out = open(dir_prefix + '/' + 'seqs.fna.tmp', 'w+')
     if record_qual_scores:
         qual_out = open(dir_prefix + '/' + 'seqs_filtered.qual', 'w+')
     else:
@@ -1097,14 +1321,15 @@ def preprocess(fasta_files, qual_files, mapping_file,
     '''log_stats, pre_lens, post_lens = check_seqs(fasta_out, fasta_files, 
         starting_ix, valid_map, qual_mappings, filters, barcode_len,
         primer_seq_len, keep_primer, keep_barcode, barcode_type, max_bc_errors,
-        remove_unassigned) '''
+        retain_unassigned_reads) '''
     log_stats, pre_lens, post_lens = check_seqs(fasta_out, fasta_files, 
         starting_ix, valid_map, qual_mappings, filters, barcode_len,
         keep_primer, keep_barcode, barcode_type, max_bc_errors,
-        remove_unassigned, attempt_bc_correction,
-        primer_seqs_lens, all_primers, max_primer_mm, disable_primers,
+        retain_unassigned_reads, attempt_bc_correction,
+        primer_seqs_lens, all_primers, max_primer_mm, disable_primer_check,
         reverse_primers, rev_primers, qual_out, qual_score_window,
-        discard_bad_windows, min_qual_score, min_seq_len)
+        discard_bad_windows, min_qual_score, min_seq_len,
+        median_length_filtering, added_demultiplex_field)
 
     # Write log file
     log_file = open(dir_prefix + '/' + "split_library_log.txt", 'w+')
