@@ -1,11 +1,11 @@
 #!/usr/bin/env python
 
 __author__ = "Daniel McDonald"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 __credits__ = ["Rob Knight", "Daniel McDonald", "Greg Caporaso", 
 "Justin Kuczynski","Jens Reeder","Catherine Lozupone"] #remember to add yourself if you make changes
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -21,17 +21,21 @@ from os import getenv, makedirs
 from operator import itemgetter
 from os.path import abspath, exists, dirname, join, isdir
 from collections import defaultdict
-from optparse import make_option
+import gzip
 import sys
 import os
 from copy import deepcopy
+from datetime import datetime
+from subprocess import Popen, PIPE, STDOUT
 from numpy import min, max, median, mean
 import numpy
 from numpy.ma import MaskedArray
 from numpy.ma.extras import apply_along_axis
 from numpy import array, zeros, argsort, shape, vstack,ndarray, asarray, \
-        float, where, isnan
-from cogent import LoadSeqs, Sequence
+        float, where, isnan, mean, std
+        
+from cogent.util.dict2d import Dict2D
+from cogent import LoadSeqs, Sequence,DNA
 from cogent.cluster.procrustes import procrustes
 from cogent.core.alignment import Alignment
 from cogent.core.moltype import MolType, IUPAC_DNA_chars, IUPAC_DNA_ambiguities,\
@@ -39,23 +43,27 @@ from cogent.core.moltype import MolType, IUPAC_DNA_chars, IUPAC_DNA_ambiguities,
 from cogent.data.molecular_weight import DnaMW
 from cogent.core.sequence import DnaSequence
 from cogent.app.blast import Blastall
-from cogent.app.util import get_tmp_filename
+from cogent.app.util import get_tmp_filename as cogent_get_tmp_filename, FilePath
 from cogent.parse.blast import BlastResult
 from cogent.parse.fasta import MinimalFastaParser
+from cogent.parse.fastq import MinimalFastqParser
 from cogent.util.misc import remove_files
 from cogent.util.dict2d import Dict2D
 from cogent.app.formatdb import build_blast_db_from_fasta_path,\
     build_blast_db_from_fasta_file
 from cogent import LoadSeqs
-from cogent.util.misc import (parse_command_line_parameters, 
-                                     create_dir, 
-                                     handle_error_codes)
+from cogent.util.misc import (create_dir, 
+                              handle_error_codes)
+from qiime import __version__ as qiime_library_version
+from cogent.util.option_parsing import (parse_command_line_parameters,
+                                        make_option)
 from qiime.parse import (parse_otu_table,
                          parse_qiime_config_files,
                          parse_coords,
                          parse_newick,
                          PhyloNode,
-                         parse_mapping_file)
+                         parse_mapping_file,
+                         parse_denoiser_mapping)
 from qiime.format import format_otu_table
 
 class TreeMissingError(IOError):
@@ -73,6 +81,10 @@ class AlignmentMissingError(IOError):
 class MissingFileError(IOError):
     pass
 
+class FileFormatError(IOError):
+    """Exception for wrong file format"""
+    pass
+
 def make_safe_f(f, allowed_params):
     """Make version of f that ignores extra named params."""
     def inner(*args, **kwargs):
@@ -265,6 +277,28 @@ def get_qiime_scripts_dir():
     
     return result
     
+def get_qiime_temp_dir():
+    """ Returns the temp directory that should be used by QIIME scripts
+    
+    """
+    qiime_config = load_qiime_config()
+    qiime_config_value = qiime_config['temp_dir']
+    if qiime_config_value != None:
+        result = qiime_config_value
+    else:
+        result = '/tmp/'
+    return result
+    
+def get_tmp_filename(tmp_dir=None, prefix="tmp", suffix=".txt",
+    result_constructor=FilePath):
+    """ Wrap cogent.app.util.get_tmp_filename to modify the default tmp_dir """
+    if tmp_dir == None:
+        tmp_dir = get_qiime_temp_dir()
+    return cogent_get_tmp_filename(tmp_dir=tmp_dir,
+                                   prefix=prefix,
+                                   suffix=suffix,
+                                   result_constructor=result_constructor)
+    
 def load_qiime_config():
     """Return default parameters read in from file"""
     
@@ -392,26 +426,58 @@ def split_fasta_on_sample_ids_to_dict(seqs):
         except KeyError:
             result[sample_id] = [(seq_id,seq)]
     return result
+
+def write_seqs_to_fasta(fp,seqs,write_mode='w'):
+    """Write seqs to fp with specified write mode ('a' or 'w')
     
-def split_fasta_on_sample_ids_to_files(seqs,output_dir):
+        seqs: list of (seq_id,seq) tuples, as obtained from
+         MinimalFastaParser
+    """
+    f = open(fp,write_mode)
+    for s in seqs:
+        f.write('>%s\n%s\n' % (s))
+    f.close()
+
+def split_fasta_on_sample_ids_to_files(seqs,
+                                       output_dir,
+                                       per_sample_buffer_size=500):
     """ output of split_fasta_on_sample_ids to fasta in specified output_dir
     
         seqs: (seq_id,seq) pairs, as generated by MinimalFastaParser
         output_dir: string defining directory where output should be 
          written, will be created if it doesn't exist
-    
+         
+        This function takes a buffered approach to writing files to 
+        avoid hitting errors arising from too many files being open
+        when working with large numbers of samples ids (e.g. > 1024 on linux)
     """
     create_dir(output_dir)
     file_lookup = {}
     for sample_id,seq_id,seq in split_fasta_on_sample_ids(seqs):
+        # grab or create the list corresponding to the current sample id
         try:
-            file_lookup[sample_id].write('>%s\n%s\n' % (seq_id,seq))
+            current_seqs = file_lookup[sample_id][1]
         except KeyError:
-            file_lookup[sample_id] = open('%s/%s.fasta' % 
-                                          (output_dir,sample_id),'w')
-            file_lookup[sample_id].write('>%s\n%s\n' % (seq_id,seq))
-    for file_handle in file_lookup.values():
-        file_handle.close()
+            current_fp = '%s/%s.fasta' % (output_dir,sample_id)
+            if exists(current_fp):
+                raise IOError,\
+                 (" %s already exists. Will not perform split -- remove this"
+                  " file or specify a different output directory." % current_fp)
+            current_seqs = list()
+            file_lookup[sample_id] = [current_fp,current_seqs]
+        
+        # append the current sequence to the current seqs list
+        current_seqs.append((seq_id,seq))
+        # compare current_seqs length to the buffer size, and write
+        # if it has hit the buffer size
+        if len(current_seqs) == per_sample_buffer_size:
+            current_fp = file_lookup[sample_id][0]
+            write_seqs_to_fasta(current_fp,current_seqs,write_mode='a')
+            # reset the current seqs buffer
+            file_lookup[sample_id][1] = list()
+    
+    for current_fp,current_seqs in file_lookup.values():
+        write_seqs_to_fasta(current_fp,current_seqs,write_mode='a')
     return None
 
 def compute_seqs_per_library_stats(otu_f):
@@ -422,7 +488,36 @@ def compute_seqs_per_library_stats(otu_f):
         
     return min(counts), max(counts), median(counts), mean(counts),\
      dict(zip(sample_ids,counts))
-     
+
+def median_absolute_deviation(x):
+    """ compute the median of the absolute deviations from the median """
+    x = asarray(x)
+    median_x = median(x)
+    median_abs_deviation = median(abs(x - median_x))
+    return median_abs_deviation, median_x
+
+def guess_even_sampling_depth(counts_per_sample,num_deviations=2.25):
+    """ guess a depth for even sampling 
+    
+        this is currently computed as the smallest seqs per sample 
+         count >= the median seqs per sample count - (2.25 * the median absolute 
+         deviation). 2.25 was chosen emprically by seeing how different values
+         of num_deviations resulted in a choice that was similar to what
+         I've chosen on several real OTU tables.
+    """
+    counts_per_sample.sort()
+    median_abs_dev, median_count = \
+     median_absolute_deviation(counts_per_sample)
+    min_threshold = median_count - (num_deviations * median_abs_dev)
+    for e in counts_per_sample:
+        if e >= min_threshold:
+            return e
+    raise ValueError,\
+     "No acceptable even sampling depth identified. "+\
+     "It shouldn't be possible to get here, but just in case here's the " +\
+     "counts per sample: %s" ' '.join(map(str,counts_per_sample)) 
+    
+    
 def raise_error_on_parallel_unavailable(qiime_config=None):
     """Raise error if no parallel QIIME bc user hasn't set jobs_to_start
     """
@@ -441,23 +536,38 @@ def get_options_lookup():
     qiime_config = load_qiime_config()
     result = {}
     result['fasta_as_primary_input'] =\
-     make_option('-i','--input_fasta_fp',help='path to the input fasta file')
+     make_option('-i','--input_fasta_fp',type="existing_filepath",
+      help='path to the input fasta file')
     result['otu_table_as_primary_input'] =\
-     make_option('-i','--otu_table_fp',\
+     make_option('-i','--otu_table_fp',type="existing_filepath",
       help='path to the input OTU table (i.e., the output from make_otu_table.py)')
     result['otu_map_as_primary_input'] =\
-     make_option('-i','--otu_map_fp',\
+     make_option('-i','--otu_map_fp',type="existing_filepath",
       help='path to the input OTU map (i.e., the output from pick_otus.py)')
     result['log_fp'] =\
-     make_option('-l','--log_fp',help='path to write the log file')
+     make_option('-l','--log_fp',type="new_filepath",
+     help='path to write the log file')
     result['input_fasta'] =\
-     make_option('-f','--input_fasta_fp',help='path to the input fasta file')
+     make_option('-f','--input_fasta_fp',type="existing_filepath",
+      help='path to the input fasta file')
     result['output_dir'] =\
-     make_option('-o','--output_dir',help='path to the output directory')
+     make_option('-o','--output_dir',type="new_dirpath",
+     help='path to the output directory')
     result['output_fp'] =\
-     make_option('-o','--output_fp',help='the output filepath')
+     make_option('-o','--output_fp',type="new_filepath",
+     help='the output filepath')
     result['mapping_fp'] =\
-     make_option('-m','--mapping_fp',help='the mapping filepath')
+     make_option('-m','--mapping_fp',type="existing_filepath",
+     help='the mapping filepath')
+     
+    ## Define options used by the workflow scripts
+    result['jobs_to_start_workflow'] =\
+     make_option('-O','--jobs_to_start',type='int',
+       help='Number of jobs to start. NOTE: you must also'
+       ' pass -a to run in parallel, this defines the number of'
+       ' jobs to be started if and only if -a is passed'
+       ' [default: %default]',
+       default=qiime_config['jobs_to_start'])
     
     ## Define options used by the parallel scripts
     result['jobs_to_start'] =\
@@ -466,7 +576,7 @@ def get_options_lookup():
        default=qiime_config['jobs_to_start'])
     result['poller_fp'] =\
      make_option('-P','--poller_fp',action='store',\
-       type='string',help='full path to '+\
+       help='full path to '+\
        'qiime/parallel/poller.py [default: %default]',\
        default=join(get_qiime_scripts_dir(),'poller.py'))
     result['retain_temp_files'] =\
@@ -486,7 +596,7 @@ def get_options_lookup():
         default=False)
     result['cluster_jobs_fp'] =\
      make_option('-U','--cluster_jobs_fp',
-        help='path to cluster_jobs.py script ' +\
+        help='path to cluster jobs script (defined in qiime_config) ' +\
         ' [default: %default]',\
         default=qiime_config['cluster_jobs_fp'] or\
          join(get_qiime_scripts_dir(),'start_parallel_jobs.py'))
@@ -531,17 +641,24 @@ def matrix_stats(headers_list, distmats):
 
 
 def merge_otu_tables(otu_table_f1,otu_table_f2):
-    """ Merge two otu tables with the same sample IDs
+    """ Merge two otu tables with non-overlapping sample IDs
     
         WARNING: The OTU ids must refer to the same OTUs, which
          typically only happens when OTUs were picked against a 
          reference database, as with the BLAST OTU picker.
     
     """
-    sample_ids1, otu_ids1, otu_table1, lineages1 =\
-        parse_otu_table(otu_table_f1)
-    sample_ids2, otu_ids2, otu_table2, lineages2 =\
-        parse_otu_table(otu_table_f2)
+    if isinstance(otu_table_f1, tuple):
+        sample_ids1, otu_ids1, otu_table1, lineages1 = otu_table_f1
+    else:
+        sample_ids1, otu_ids1, otu_table1, lineages1 =\
+            parse_otu_table(otu_table_f1)
+    
+    if isinstance(otu_table_f2, tuple):
+        sample_ids2, otu_ids2, otu_table2, lineages2 = otu_table_f2
+    else:
+        sample_ids2, otu_ids2, otu_table2, lineages2 =\
+            parse_otu_table(otu_table_f2)
     
     assert set(sample_ids1) & set(sample_ids2) == set(),\
      'Overlapping sample ids detected.'
@@ -930,6 +1047,306 @@ def compare_otu_maps(otu_map1, otu_map2, verbose=False):
 
     return float(wrong)/(right+wrong)
     
+def compute_days_since_epoch(day,month,year):
+    """ pass day, month, year to compute days since epoch (1/1/1970) 
+        
+        Note that full years should always be provided: 09 is a 
+        different year than 2009!
+    """
+    d = datetime(int(year),int(month),int(day))
+    epoch = datetime.utcfromtimestamp(0)
+    return (d - epoch).days
+    
+def get_interesting_mapping_fields(mapping_data,mapping_headers):
+    """ Returns headers for fields that are useful to color by in plots 
+    
+        These fields are the ones that contain greater than one value 
+         and less values than the number of entries (so for example 
+         not SampleID) 
+    """
+    result = []
+    num_samples = len(mapping_data)
+    num_cols = len(mapping_headers)
+    transposed_data = array(mapping_data).T
+    for header, datum in zip(mapping_headers, transposed_data):
+        d = set(datum)
+        len_d = len(d)
+        if len_d > 1 and len_d < num_samples:
+            result.append(header)
+    return result
+    
+def flowgram_id_to_seq_id_map(seqs):
+    """Map flowgram ids to sequence ids from a post-split_libraries fasta file
+    """
+    result = {}
+    for id_, seq in seqs:
+        fields = id_.split()
+        seq_id = id_
+        flowgram_id = fields[1]
+        result[flowgram_id] = seq_id
+    return result
+    
+def qiime_system_call(cmd):
+    """ Call cmd and return (stdout, stderr, return_value)"""
+    proc = Popen(cmd,shell=True,universal_newlines=True,\
+                 stdout=PIPE,stderr=PIPE)
+    # communicate pulls all stdout/stderr from the PIPEs to 
+    # avoid blocking -- don't remove this line!
+    stdout, stderr = proc.communicate()
+    return_value = proc.returncode
+    return stdout, stderr, return_value
+    
+def get_qiime_library_version():
+    """get QIIME version, including the svn version if applicable"""
+    qiime_dir = get_qiime_project_dir()
+    qiime_version = qiime_library_version
+    cmd_call = 'svn info %s | egrep "Revision: "' % (qiime_dir)
+    o,e,r = qiime_system_call(cmd_call)
+    svn_revision = o.replace("Revision: ","").strip()
+    
+    if len(svn_revision) > 0:
+        return '%s, svn revision %s' % (__version__, svn_revision)
+    else:
+        return '%s' % __version__
+
+def inflate_denoiser_output(centroid_seqs,singleton_seqs,denoiser_map,raw_seqs):
+    """Expand denoiser fasta files based on denoiser map
+    
+        The inflation process works as follows: write each centroid 
+         sequence n times, where n is the number of reads in that 
+         cluster, and write each singleton once. While writing these 
+         out map back to original sequence identifiers.
+         
+        The seqs objects passed in are lists of (seq_id, seq) tuples,
+         as returned from MinimalFastaParser.
+
+    
+    """
+    id_lookup = parse_denoiser_mapping(denoiser_map)
+    flowgram_to_seq_id_lookup = flowgram_id_to_seq_id_map(raw_seqs)
+    for id_, seq in centroid_seqs:
+        #centroid headers look like
+        #>FZTHQMS01E140G | cluster size: 4353
+        id, cluster_size_str = id_.split(' | ')
+        cluster_member_ids = id_lookup[id]
+        for c in cluster_member_ids:
+            yield flowgram_to_seq_id_lookup[c], seq
+    
+    for id_, seq in singleton_seqs:
+        yield flowgram_to_seq_id_lookup[id_], seq
+    
+    return
+    
+## Functions for counting sequences in fasta files
+
+def count_seqs(fasta_filepath):
+    """ Count the sequences in fasta_filepath 
+    
+        fasta_filepath: string indicating the full path to the file
+    """
+    # Open the file and pass it to py_count_seqs_from_file -- wrapping
+    # this makes for easier unit testing
+    return count_seqs_from_file(open(fasta_filepath))
+
+def count_seqs_from_file(fasta_file):
+    """Return number of sequences in fasta_file (no format checking performed)
+    
+        fasta_file: an open file object
+    
+    """
+    result = 0
+    lens = []
+    for seq_id,seq in MinimalFastaParser(fasta_file):
+        result += 1
+        lens.append(len(seq))
+    if result == 0:
+        return result, None, None
+    else:
+        return result, mean(lens), std(lens)
+        
+
+def count_seqs_in_filepaths(fasta_filepaths,seq_counter=count_seqs):
+    """ Wrapper to apply seq_counter to fasta_filepaths
+    
+        fasta_filepaths: list of one or more fasta filepaths
+        seq_counter: a function which takes a single filepath 
+         and returns the count of the number of sequences
+         (default: count_seqs) -- this is parameterized to
+         facilitate unit testing
+    """
+    total = 0
+    counts = []
+    inaccessible_filepaths = []
+    # iterate over the input files
+    for fasta_filepath in fasta_filepaths:
+        try:
+            # get the count of sequences in the current file
+            current_count = seq_counter(fasta_filepath)
+            # store it
+            counts.append((current_count,fasta_filepath))
+            # and increment the total count
+            total += current_count[0]
+        except IOError:
+            # if the file couldn't be open, keep track of the filepath
+            inaccessible_filepaths.append(fasta_filepath)
+    
+    return counts, total, inaccessible_filepaths
+    
+## End functions for counting sequences in fasta files
+
+def get_top_fastq_two_lines(open_file):
+    """ This function returns the first 4 lines of the open fastq file
+    """
+    line1 = open_file.readline()
+    line2 = open_file.readline()
+    line3 = open_file.readline()
+    line4 = open_file.readline()
+    open_file.seek(0)
+    return line1, line2, line3, line4
+    
+
+def get_split_libraries_fastq_params_and_file_types(fastq_fps,mapping_fp):
+    """ The function takes a list of open fastq files and a mapping file, then 
+        returns a recommended parameters string for split_libraries_fastq
+    """
+    #parse the mapping
+    data, headers, run_description= parse_mapping_file(open(mapping_fp,'U'))
+    
+    #determine the which column of mapping file is the BarcodeSequence
+    for i,col_head in enumerate(headers):
+        if col_head=='BarcodeSequence':
+            barcode_column=i
+            
+    #create a set of barcodes for easier lookup
+    barcode_mapping_column=set(zip(*data)[barcode_column])
+    
+    #create set of reverse complement barcodes from mapping file
+    revcomp_barcode_mapping_column=[]
+    for i in barcode_mapping_column:
+        revcomp_barcode_mapping_column.append(DNA.rc(i))
+    revcomp_barcode_mapping_column=set(revcomp_barcode_mapping_column)
+    
+    # get the filenames and sort them, so the file1 corresponds to file2
+    fastq_fps.sort()
+    
+    # get the len of the sequence in each of the files, so we can determine
+    # which file is the sequence file and which is the barcode sequence
+    get_file_type_info={}
+    for fastq_file in fastq_fps:
+        fastq_fp=open(fastq_file)
+        file_lines=get_top_fastq_two_lines(fastq_fp)
+        parsed_fastq=MinimalFastqParser(file_lines,strict=False)
+        for i,seq_data in enumerate(parsed_fastq):
+            if i==0:
+                get_file_type_info[fastq_file]=len(seq_data[1])
+            else:
+                break
+        fastq_fp.close()
+
+    # iterate over the sequence lengths and assign each file to either 
+    # a sequence list or barcode list
+    barcode_files=[]
+    sequence_files=[]
+    for i in range(0,len(fastq_fps),2):
+        if get_file_type_info[fastq_fps[i]]<get_file_type_info[fastq_fps[i+1]]:
+            barcode_files.append(fastq_fps[i])
+            sequence_files.append(fastq_fps[i+1])
+        else:
+            barcode_files.append(fastq_fps[i+1])
+            sequence_files.append(fastq_fps[i])
+    
+    # count the number of barcode matches in the forward and reverse direction
+    # to determine if the rev_comp_barcode option needs passed
+    fwd_count=0
+    rev_count=0
+    for bfile in barcode_files:
+        fastq_fp=open(bfile)
+        parsed_fastq=MinimalFastqParser(fastq_fp,strict=False)
+        for bdata in parsed_fastq:
+            if bdata[1] in barcode_mapping_column:
+                fwd_count+=1
+            elif bdata[1] in revcomp_barcode_mapping_column:
+                rev_count+=1
+        fastq_fp.close()
+    
+    # determine which barcode direction is correct
+    if rev_count > fwd_count:
+        barcode_orientation='--rev_comp_barcode'
+    else:
+        barcode_orientation=''
+    
+    #generate the string to use in command call to split_libraries_fastq
+    split_lib_str='-i %s -b %s %s' % (','.join(sequence_files),
+                                      ','.join(barcode_files),
+                                      barcode_orientation)
+    return split_lib_str
 
 
-
+def iseq_to_qseq_fields(line,barcode_in_header,barcode_length,barcode_qual_c='b'):
+    """ Split an Illumina sequence line into qseq fields"""
+    record = line.strip().split(':')
+    rec_0_1, rec_0_2 = record[0].split('_')
+    rec_4_1, rec_4_23 = record[4].split('#')
+    rec_4_2, rec_4_3 = rec_4_23.split('/')
+    if barcode_in_header:   
+        barcode = rec_4_2[:barcode_length]
+        sequence = record[5]
+        barcode_qual = barcode_qual_c*barcode_length
+        sequence_qual = record[6]
+    else:
+        barcode = record[5][:barcode_length]
+        sequence = record[5][barcode_length:]
+        barcode_qual = record[6][:barcode_length]
+        sequence_qual = record[6][barcode_length:]
+    return (rec_0_1,rec_0_2,record[1],record[2],record[3],\
+            rec_4_1,rec_4_2,rec_4_3), sequence, sequence_qual,\
+            barcode,barcode_qual
+            
+def gzip_open(fp):
+    return gzip.open(fp,'rb')
+    
+def make_compatible_distance_matrices(dm1,dm2,lookup=None):
+    """ Intersect distance matrices and sort the values """
+    dm1_ids = dm1[0]
+    dm1_data = dm1[1]
+    dm2_ids = dm2[0]
+    dm2_data = dm2[1]
+    if lookup:
+        try:
+            dm1_ids = [lookup[e] for e in dm1_ids]
+            dm2_ids = [lookup[e] for e in dm2_ids]
+        except KeyError,e:
+            raise KeyError,\
+             ("All entries in both DMs must be in "
+              "lookup if a lookup is provided. Missing: %s" % str(e))
+    order = [e for e in dm1_ids if e in dm2_ids]
+    
+    # create Dict2D from dm1
+    d1 = {}
+    for i,r in enumerate(dm1_ids):
+        d1[r] = {}
+        for j,c in enumerate(dm1_ids):
+            d1[r][c] = dm1_data[i,j]
+    result1 = Dict2D(data=d1,RowOrder=order,ColOrder=order)
+    # remove entries not in order
+    result1.purge()
+    # return 2d list in order
+    result1 = array(result1.toLists())
+    
+    # create Dict2D from dm2
+    d2 = {}
+    for i,r in enumerate(dm2_ids):
+        d2[r] = {}
+        for j,c in enumerate(dm2_ids):
+            d2[r][c] = dm2_data[i,j]
+    result2 = Dict2D(data=d2,RowOrder=order,ColOrder=order)
+    # remove entries not in order
+    result2.purge()
+    # return 2d list in order
+    result2 = array(result2.toLists())
+    
+    return (order,result1), (order,result2)
+    
+def get_rdp_jarpath():
+    """ Return jar file name for RDP classifier ($RDP_JAR_PATH)"""
+    return getenv('RDP_JAR_PATH')
\ No newline at end of file
