@@ -6,7 +6,7 @@ __author__ = "Jesse Stombauhg"
 __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Jesse Stombaugh"]
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Jesse Stombaugh"
 __email__ = "jesse.stombaugh@colorado.edu"
 __status__ = "Release"
@@ -38,5 +38,8 @@ script_info['optional_options']=[]
 
 script_info['version'] = __version__
 
+def main():
+    print deprecation_message
 
-print deprecation_message
\ No newline at end of file
+if __name__ == "__main__":
+    main()
\ No newline at end of file
