@@ -3,22 +3,22 @@
 from __future__ import division
 
 __author__ = "Jens Reeder"
-__copyright__ = "Copyright 2010, The QIIME project"
+__copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Greg Caporaso", "Jens Reeder"]
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
  
 
-from optparse import make_option
+from qiime.util import make_option
 from os import popen, system, makedirs, mkdir
 from os.path import split, splitext, join
 from subprocess import check_call, CalledProcessError
 from shutil import copy
 
-from cogent.app.util import get_tmp_filename
+from qiime.util import get_tmp_filename
 from cogent.app.formatdb import build_blast_db_from_fasta_path
 from cogent.parse.fasta import MinimalFastaParser
 
