@@ -3,16 +3,17 @@
 from __future__ import division
 
 __author__ = "Jens Reeder"
-__copyright__ = "Copyright 2010, The QIIME project"
-__credits__ = ["Jens Reeder","Dan Knights", "Antonio Gonzalez Pena"]
+__copyright__ = "Copyright 2011, The QIIME Project"
+__credits__ = ["Jens Reeder","Dan Knights", "Antonio Gonzalez Pena",
+    "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
 
-from optparse import make_option
-from os import access, X_OK, R_OK, W_OK, getenv
+from qiime.util import make_option
+from os import access, X_OK, R_OK, W_OK, getenv, environ
 from os.path import isdir, exists, split
 from sys import platform, version as python_version, executable
 from shutil import rmtree
@@ -20,11 +21,16 @@ from subprocess import Popen, PIPE, STDOUT
 
 from cogent.util.unit_test import TestCase, main as test_main
 from cogent.util.misc import app_path, get_random_directory_name
-from cogent.app.util import ApplicationNotFoundError
+from cogent.app.util import ApplicationNotFoundError, ApplicationError
 
 from qiime.parse import parse_qiime_config_file
-from qiime.util import load_qiime_config, get_qiime_project_dir, parse_command_line_parameters
-from qiime import __version__ as qiime_lib_version
+from qiime.util import (load_qiime_config, 
+                        get_qiime_project_dir, 
+                        parse_command_line_parameters,
+                        get_qiime_library_version,
+                        get_rdp_jarpath)
+from qiime.denoiser.utils import check_flowgram_ali_exe
+
 from cogent import __version__ as pycogent_lib_version
 from numpy import __version__ as numpy_lib_version
 try:
@@ -36,11 +42,6 @@ try:
 except ImportError:
     pynast_lib_version = "Not installed."
 
-try:
-    from Denoiser import __version__ as denoiser_version
-except ImportError:
-    denoiser_version = "Not installed."
-
 script_info = {}
 script_info['brief_description']= """Print out the qiime config settings."""
 script_info['script_description'] = """A simple scripts that prints out the qiime config settings and does some sanity checks."""
@@ -54,7 +55,7 @@ script_info['script_usage'].append(
 script_info['output_description'] = """This prints the qiime_config to stdout."""
 script_info['version'] = __version__
 script_info['help_on_no_arguments'] = False
-
+script_info['required_options']=[]
 script_info['optional_options'] = [\
     make_option('-t','--test', action='store_true',
                 dest='test', default = False,
@@ -106,36 +107,55 @@ class Qiime_config(TestCase):
 
         test_qiime_config_variable("blastall_fp", self.config, self, X_OK)
 
-    def test_rdp_classifier_fp(self):
-        """rdp_classifier is set to a valid path"""
-            
-        test_qiime_config_variable("rdp_classifier_fp", self.config, self)
+    def test_ampliconnoise_install(self):
+        """ AmpliconNoise install looks sane."""
+        url = "http://www.qiime.org/install/install.html#ampliconnoise-install"
+        
+        pyro_lookup_file = getenv('PYRO_LOOKUP_FILE')
+        self.assertTrue(pyro_lookup_file != None,
+         "$PYRO_LOOKUP_FILE variable is not set. See %s for help." % url)
+        self.assertTrue(exists(pyro_lookup_file),
+         "$PYRO_LOOKUP_FILE variable is not set to an existing filepath.")
+         
+        seq_lookup_file = getenv('SEQ_LOOKUP_FILE')
+        self.assertTrue(seq_lookup_file != None,
+         "$SEQ_LOOKUP_FILE variable is not set. See %s for help." % url)
+        self.assertTrue(exists(seq_lookup_file),
+         "$SEQ_LOOKUP_FILE variable is not set to an existing filepath.")
+         
+        self.assertTrue(app_path("SplitKeys.pl"),
+         "AmpliconNoise Scripts directory doesn't seem to be in $PATH."+\
+         " See %s for help." % url)
+         
+        self.assertTrue(app_path("Perseus"),
+         "AmpliconNoise bin directory doesn't seem to be in $PATH."+\
+         " See %s for help." % url)
         
     def test_pynast_template_alignment_fp(self):
-        """pynast_template_alignment is set to a valid path"""
+        """pynast_template_alignment, if set, is set to a valid path"""
             
         test_qiime_config_variable("pynast_template_alignment_fp",
                                    self.config, self)
             
     def test_pynast_template_alignment_blastdb_fp(self):
-        """pynast_template_alignment_blastdb is set to a valid path"""
+        """pynast_template_alignment_blastdb, if set, is set to a valid path"""
             
         test_qiime_config_variable("pynast_template_alignment_blastdb_fp",
                                    self.config, self)
     def test_pynast_template_alignment_blastdb_fp(self):
-        """pynast_template_alignment_blastdb is set to a valid path"""
+        """pynast_template_alignment_blastdb, if set, is set to a valid path"""
         
         test_qiime_config_variable("pynast_template_alignment_blastdb_fp",
                                    self.config, self)
         
     def test_template_alignment_lanemask_fp(self):
-        """template_alignment_lanemask is set to a valid path"""
+        """template_alignment_lanemask, if set, is set to a valid path"""
             
         test_qiime_config_variable("template_alignment_lanemask_fp",
                                    self.config, self)
     
     def test_qiime_scripts_dir(self):
-        """qiime_scripts_dir is set to a valid path"""
+        """qiime_scripts_dir, if set, is set to a valid path"""
 
         scripts_dir = self.config["qiime_scripts_dir"]
         
@@ -147,9 +167,23 @@ class Qiime_config(TestCase):
         else:
             pass
             #self.fail("scripts_dir is not set.")
-                        
+
+    def test_temp_dir(self):
+        """temp_dir, if set, is set to a valid path"""
+
+        temp_dir = self.config["temp_dir"]
+        
+        if temp_dir:
+            self.assertTrue(exists(temp_dir),
+                            "temp_dir does not exist: %s" % temp_dir)
+            self.assertTrue(isdir(temp_dir),
+                            "temp_dir is not a directory: %s" % temp_dir)
+        else:
+            pass
+            #self.fail("temp_dir is not set.")
+
     def test_working_dir(self):
-        """working_dir is set to a valid path"""
+        """working_dir, if set, is set to a valid path"""
 
         working_dir = self.config["working_dir"]
         
@@ -249,6 +283,8 @@ class Qiime_config(TestCase):
         version_string = stdout.strip().split('Python')[-1].strip()
         try:
             version = tuple(map(int,version_string.split('.')))
+            if len(version) == 2:
+                version = tuple(version[0],version[1],0)
             pass_test = (version >= min_acceptable_version and version < min_unacceptable_version)
         except ValueError:
             pass_test = False
@@ -259,6 +295,63 @@ class Qiime_config(TestCase):
             '.'.join(map(str,min_unacceptable_version)),
             version_string))
 
+    def test_numpy_suported_version(self):
+        """numpy version is supported """
+        min_acceptable_version = (1,3,0)
+        min_unacceptable_version = (1,5,1)
+        try:
+            from numpy import __version__ as numpy_lib_version
+            version = tuple(map(int,numpy_lib_version.split('.')))
+            pass_test = (version >= min_acceptable_version and version < min_unacceptable_version)
+            version_string = str(numpy_lib_version)
+        except ImportError:
+            pass_test = False
+            version_string = "Not installed"
+        self.assertTrue(pass_test,\
+         "Unsupported numpy version. Must be >= %s and < %s , but running %s." \
+         % ('.'.join(map(str,min_acceptable_version)),
+            '.'.join(map(str,min_unacceptable_version)),
+            version_string))
+
+    def test_matplotlib_suported_version(self):
+        """maptplotlib version is supported """
+        min_acceptable_version = (0,98,5,3)
+        min_unacceptable_version = (0,98,5,4)
+        try:
+            from matplotlib import __version__ as matplotlib_lib_version
+            version = tuple(map(int,matplotlib_lib_version.split('.')))
+            pass_test = (version >= min_acceptable_version and version < min_unacceptable_version)
+            version_string = str(matplotlib_lib_version)
+        except ImportError:
+            pass_test = False
+            version_string = "Not installed"
+        self.assertTrue(pass_test,\
+         "Unsupported matplotlib version. Must be >= %s and < %s , but running %s." \
+         % ('.'.join(map(str,min_acceptable_version)),
+            '.'.join(map(str,min_unacceptable_version)),
+            version_string))
+            
+    def test_pynast_suported_version(self):
+        """pynast version is supported """
+        min_acceptable_version = (1,1)
+        min_unacceptable_version = (1,2)
+        try:
+            from pynast import __version__ as pynast_lib_version
+            version = pynast_lib_version.split('.')
+            if version[-1][-4:]=='-dev':
+                 version[-1] = version[-1][:-4]
+            version = tuple(map(int,version))
+            pass_test = (version >= min_acceptable_version and version < min_unacceptable_version)
+            version_string = str(pynast_lib_version)
+        except ImportError:
+            pass_test = False
+            version_string = "Not installed"
+        self.assertTrue(pass_test,\
+         "Unsupported matplotlib version. Must be >= %s and < %s , but running %s." \
+         % ('.'.join(map(str,min_acceptable_version)),
+            '.'.join(map(str,min_unacceptable_version)),
+            version_string))
+
     def test_blast_supported_version(self):
         """blast is in path and version is supported """
         acceptable_version = (2,2,22)
@@ -282,7 +375,7 @@ class Qiime_config(TestCase):
          
     def test_FastTree_supported_version(self):
         """FastTree is in path and version is supported """
-        acceptable_version = (2,1,0)
+        acceptable_version = (2,1,3)
         self.assertTrue(app_path('FastTree'),
          "FastTree not found. This may or may not be a problem depending on "+\
          "which components of QIIME you plan to use.")
@@ -391,24 +484,17 @@ class Qiime_config(TestCase):
         rmtree(tmp_dir)
          
     def test_denoiser_supported_version(self):
-        """denoiser is in path and version is supported """
-        acceptable_version = (0,851)
+        """denoiser aligner is ready to use """
+
+        pass_test = True
         try:
-            from Denoiser import __version__ as version_string
-            try:
-                version = tuple(map(int,version_string.split('.')))
-                pass_test = version == acceptable_version
-            except ValueError:
-                pass_test = False
-                version_string = stdout
-        except ImportError:
-            self.assertTrue(False,"denoiser not found. This may or may not "+\
-             "be a problem depending on which components of QIIME you plan to use.")
-            version_string = "Not installed."
+            check_flowgram_ali_exe()
+        except (ApplicationNotFoundError, ApplicationError):
             pass_test = False
-        self.assertTrue(pass_test,\
-         "Unsupported denoiser version. %s is required, but running %s." \
-         % ('.'.join(map(str,acceptable_version)), version_string))
+            
+        self.assertTrue(pass_test, "Denoiser flowgram aligner not found or not executable."+\
+                            "This may or may not be a problem depending on "+\
+                            "which components of QIIME you plan to use.")
         
     def test_raxmlHPC_supported_version(self):
         """raxmlHPC is in path and version is supported """
@@ -452,26 +538,12 @@ class Qiime_config(TestCase):
          "Unsupported clearcut version. %s is required, but running %s." \
          % ('.'.join(map(str,acceptable_version)), version_string))
         
-    def test_pyronoise2_supported_version(self):
-        """pyrnoise2 is in path and version is supported """
-        self.assertTrue(app_path('PCluster'),
-         "Pyronoise not found. This may or may not be a problem depending on "+\
-         "which components of QIIME you plan to use.") 
-        # pyrnoise2 does not have a version print in their program
-        
     def test_cdhit_supported_version(self):
         """cd-hit is in path and version is supported """
         self.assertTrue(app_path('cd-hit'),
          "cd-hit not found. This may or may not be a problem depending on "+\
          "which components of QIIME you plan to use.")
         # cd-hit does not have a version print in their program
-        
-    def test_dotur_supported_version(self):
-        """dotur is in path and version is supported """
-        self.assertTrue(app_path('dotur'),
-         "dotur not found. This may or may not be a problem depending on "+\
-         "which components of QIIME you plan to use.")
-        # dotur does not have a version print in their program
     
     #def test_rdp_classifier_supported_version(self):
     #    """rdp_classifier is in path and version is supported """
@@ -511,12 +583,16 @@ def test_qiime_config_variable(variable, qiime_config, test,
     test.assertTrue(access(fp, access_var),
                     "%s is not %s: %s" % (variable, modes[access_var], fp))
 
-
-
 if __name__ == "__main__":
     option_parser, opts, args = parse_command_line_parameters(**script_info)
 
     qiime_config = load_qiime_config()
+    
+    rdp_jarpath = get_rdp_jarpath()
+    if rdp_jarpath == None:
+        rdp_version = "Not installed."
+    else:
+        rdp_version = split(rdp_jarpath)[1]
 
     system_info = [
      ("Platform", platform),
@@ -532,10 +608,10 @@ if __name__ == "__main__":
      ("PyCogent version", pycogent_lib_version),
      ("NumPy version", numpy_lib_version),
      ("matplotlib version", matplotlib_lib_version),
-     ("QIIME library version", qiime_lib_version),
+     ("QIIME library version", get_qiime_library_version()),
      ("QIIME script version", __version__),
      ("PyNAST version (if installed)", pynast_lib_version),
-     ("Denoiser version (if installed)", denoiser_version)]
+     ("RDP Classifier version (if installed)", rdp_version)]
     max_len =  max([len(e[0]) for e in version_info])
     print "\nDependency versions"
     print  "===================" 
