@@ -2,11 +2,11 @@
 #file test_parse.py
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2010, The QIIME Project"
+__copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Rob Knight", "Justin Kuczynski", "Greg Caporaso",\
-                "Cathy Lozupone", "Jens Reeder"] #remember to add yourself
+                "Cathy Lozupone", "Jens Reeder", "Daniel McDonald"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -15,7 +15,7 @@ from numpy import array, nan
 from StringIO import StringIO
 from cogent.util.unit_test import TestCase,main
 from cogent.util.misc import remove_files
-from cogent.app.util import get_tmp_filename
+from qiime.util import get_tmp_filename
 from qiime.parse import (group_by_field, group_by_fields, 
     parse_distmat, parse_rarefaction_record, parse_rarefaction, parse_coords, 
     parse_otu_table, make_envs_dict, fields_to_dict, parse_rarefaction_fname,
@@ -25,7 +25,8 @@ from qiime.parse import (group_by_field, group_by_fields,
     parse_metadata_state_descriptions, parse_rarefaction_data,
     parse_illumina_line, parse_qual_score, parse_qual_scores, QiimeParseError,
     parse_newick,parse_trflp,parse_taxa_summary_table, parse_prefs_file,
-    parse_mapping_file_to_dict, mapping_file_to_dict)
+    parse_mapping_file_to_dict, mapping_file_to_dict, MinimalQualParser,
+    parse_denoiser_mapping)
 
 class TopLevelTests(TestCase):
     """Tests of top-level functions"""
@@ -68,7 +69,9 @@ class TopLevelTests(TestCase):
         self.expected_lineages1 = expected_lineages1
         self.taxa_summary1 = taxa_summary1
         self.taxa_summary1_expected = taxa_summary1_expected
+        self.otu_table1_floats=otu_table1_floats
         self.files_to_remove = []
+        self.denoiser_mapping1 = denoiser_mapping1.split('\n')
     
     def tearDown(self):
         remove_files(self.files_to_remove)
@@ -467,6 +470,21 @@ eigvals\t4.94\t1.79\t1.50
                       [1722,4903,17], [589,2074,34]]),
                self.expected_lineages1)
         self.assertEqual(obs, exp)
+    
+    def test_parse_otu_table_floats_in_table(self):
+        """parse_otu_table functions using an OTU table containing floats
+           but cast as int....this will automatically cast into floats"""
+           
+        data = self.otu_table1_floats
+        data_f = (data.split('\n'))
+        obs = parse_otu_table(data_f)
+        exp = (['Fing','Key','NA'],
+               ['0','1','2','3','4'],
+               array([[19111.0,44536.0,42.0],[1216.0,3500.0,6.0],
+                      [1803.0,1184.0,2.0],[1722.1,4903.2,17.0],
+                      [589.6,2074.4,34.5]]),
+               self.expected_lineages1)
+        self.assertEqual(obs, exp)  
         
     def test_parse_otu_table_float_counts(self):
         """parse_otu_table should return correct result from small table"""
@@ -590,6 +608,7 @@ eigvals\t4.94\t1.79\t1.50
         OTU_sample_info, all_sample_names = parse_sample_mapping(lines)
         self.assertEqual(OTU_sample_info, {'OTU2': {'sample1': '1', 'sample3': '0', 'sample2': '1'}, 'OTU1': {'sample1': '1', 'sample3': '1', 'sample2': '0'}})
         self.assertEqual(all_sample_names, set(['sample1', 'sample3', 'sample2']))
+
     def test_sample_mapping_to_otu_table(self):
         """sample_mapping_to_otu_table works"""
         lines = self.SampleMapping
@@ -729,6 +748,18 @@ eigvals\t4.94\t1.79\t1.50
         scores2= StringIO('>a\n5 10 5\n12\n>b\n30 40')
         self.assertEqual(parse_qual_scores([scores, scores2]),
             {'x':[5,10,5,12],'y':[30,40],'a':[5,10,5,12],'b':[30,40]})
+    
+    def test_MinimalQualParser(self):
+        """MinimalQualParser should yield (id_, quals)"""
+        scores = ['>x','5 10 5','12',
+                  '>y','30 40',
+                  '>a','5 10 5','12',
+                  '>b','30 40']
+        gen = MinimalQualParser(scores)
+        self.assertEqual(list(gen), [('x',[5,10,5,12]),
+                                     ('y',[30,40]),
+                                     ('a',[5,10,5,12]),
+                                     ('b',[30,40])])
 
     def test_parse_trflp(self):
         """ should return a header and otu_table lists"""
@@ -798,6 +829,16 @@ Sample 5	25
         self.assertEqual(samples, samples_exp)
         self.assertEqual(otus, otus_exp)
         self.assertEqual(data, data_exp)
+
+
+    def test_parse_denoiser_mapping(self):
+        """ parse_denoiser_mapping creates {} from denoiser mapping file
+        """
+        actual = parse_denoiser_mapping(self.denoiser_mapping1)
+        expected = {'Read1':['Read1','Read4','Read5 some comment'],
+                    'Read2':['Read2'],
+                    'Read3':['Read3','Read6']}
+        self.assertEqual(actual,expected)
         
 
 illumina_read1 = """HWI-6X_9267:1:1:4:1699#ACCACCC/1:TACGGAGGGTGCGAGCGTTAATCGCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCGAAAAAAAAAAAAAAAAAAAAAAA:abbbbbbbbbb`_`bbbbbb`bb^aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaDaabbBBBBBBBBBBBBBBBBBBB
@@ -835,6 +876,22 @@ OTU ID	Fing	Key	NA	Consensus Lineage
 4	589	2074	34	Bacteria; Cyanobacteria; Chloroplasts; vectors
 """
 
+otu_table1_floats = """# Some comment
+
+
+
+
+OTU ID	Fing	Key	NA	Consensus Lineage
+0	19111.0	44536.0	42.0	Bacteria; Actinobacteria; Actinobacteridae; Propionibacterineae; Propionibacterium
+# some other comment
+1	1216.0	3500.0	6.0	Bacteria; Firmicutes; Alicyclobacillaceae; Bacilli; Lactobacillales; Lactobacillales; Streptococcaceae; Streptococcus
+2	1803.0	1184.0	2.0	Bacteria; Actinobacteria; Actinobacteridae; Gordoniaceae; Corynebacteriaceae
+# comments
+#    everywhere!
+3	1722.1	4903.2	17	Bacteria; Firmicutes; Alicyclobacillaceae; Bacilli; Staphylococcaceae
+4	589.6	2074.4	34.5	Bacteria; Cyanobacteria; Chloroplasts; vectors
+"""
+
 taxa_summary1 = """#Full OTU Counts
 Taxon	Even1	Even2	Even3
 Bacteria;Actinobacteria;Actinobacteria(class);Actinobacteridae	0.0880247251673	0.0721968465746	0.081371761759
@@ -868,5 +925,10 @@ expected_lineages1 = [['Bacteria','Actinobacteria','Actinobacteridae','Propionib
 ['Bacteria','Firmicutes','Alicyclobacillaceae','Bacilli','Staphylococcaceae'],
 ['Bacteria','Cyanobacteria','Chloroplasts','vectors']]
 
+denoiser_mapping1 = """Read1:\tRead4\tRead5 some comment
+Read2:
+Read3:\tRead6"""
+
+
 if __name__ =='__main__':
     main()
