@@ -3,11 +3,11 @@
 """Tests of code for OTU picking"""
 
 __author__ = "Kyle Bittinger, Greg Caporaso"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 #remember to add yourself if you make changes
 __credits__ = ["Kyle Bittinger", "Greg Caporaso", "Rob Knight", "Jens Reeder","William Walters"] 
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -16,8 +16,9 @@ from os import remove
 
 from cogent.util.misc import create_dir
 from cogent.util.unit_test import TestCase, main
-from cogent.app.util import get_tmp_filename
-from cogent.util.misc import remove_files, revComp
+from qiime.util import get_tmp_filename, load_qiime_config
+from cogent.util.misc import remove_files
+from cogent import DNA
 from cogent.app.formatdb import build_blast_db_from_fasta_path
 from qiime.pick_otus import (CdHitOtuPicker, OtuPicker,
     MothurOtuPicker, PrefixSuffixOtuPicker, TrieOtuPicker, BlastOtuPicker,
@@ -38,6 +39,26 @@ class OtuPickerTests(TestCase):
         """Abstract OtuPicker __call__ should raise NotImplementedError"""
         p = OtuPicker({})
         self.assertRaises(NotImplementedError, p, '/path/to/seqs')
+        
+    def test_prefilter_exact_matches(self):
+        """Abstract OtuPicker _prefilter_exact_matches functions as expected
+        """
+        seqs = [('s1 comment1','ACCTTGTTACTTT'),  # three copies
+                ('s2 comment2','ACCTTGTTACTTTC'), # one copy
+                ('s3 comment3','ACCTTGTTACTTTCC'),# two copies
+                ('s4 comment4','ACCTTGTTACTTT'),
+                ('s5 comment5','ACCTTGTTACTTTCC'),
+                ('s6 comment6','ACCTTGTTACTTT')]
+        expected0 = [('QiimeExactMatch.s1','ACCTTGTTACTTT'),
+                     ('QiimeExactMatch.s2','ACCTTGTTACTTTC'),
+                     ('QiimeExactMatch.s3','ACCTTGTTACTTTCC')]
+        expected1 = {'QiimeExactMatch.s1':['s1','s4','s6'],
+                     'QiimeExactMatch.s2':['s2'],
+                     'QiimeExactMatch.s3':['s3','s5']}
+        expected = (expected0,expected1)
+        p = OtuPicker({})
+        actual = p._prefilter_exact_matches(seqs)
+        self.assertEqual(actual,expected)
 
 
 class MothurOtuPickerTests(TestCase):
@@ -99,10 +120,10 @@ class BlastOtuPickerTests(TestCase):
         ]
         
         self.ref_seqs_rc = [\
-         ('ref1',revComp('TGCAGCTTGAGCCACAGGAGAGAGAGAGCTTC')),\
-         ('ref2',revComp('ACCGATGAGATATTAGCACAGGGGAATTAGAACCA')),\
-         ('ref3',revComp('TGTCGAGAGTGAGATGAGATGAGAACA')),\
-         ('ref4',revComp('ACGTATTTTAATGGGGCATGGT')),\
+         ('ref1',DNA.rc('TGCAGCTTGAGCCACAGGAGAGAGAGAGCTTC')),\
+         ('ref2',DNA.rc('ACCGATGAGATATTAGCACAGGGGAATTAGAACCA')),\
+         ('ref3',DNA.rc('TGTCGAGAGTGAGATGAGATGAGAACA')),\
+         ('ref4',DNA.rc('ACGTATTTTAATGGGGCATGGT')),\
         ]
         
         self.seqs_fp = get_tmp_filename(
@@ -545,6 +566,7 @@ class UclustOtuPickerTests(TestCase):
 
     def setUp(self):
         # create the temporary input files
+        self.temp_dir = load_qiime_config()['temp_dir']
         self.tmp_seq_filepath1 = get_tmp_filename(\
          prefix='UclustOtuPickerTest_',\
          suffix='.fasta')
@@ -566,8 +588,16 @@ class UclustOtuPickerTests(TestCase):
         seq_file.write(dna_seqs_5)
         seq_file.close()
         
+        self.tmp_seq_filepath4 = get_tmp_filename(\
+         prefix='UclustOtuPickerTest_',\
+         suffix='.fasta')
+        seq_file = open(self.tmp_seq_filepath4,'w')
+        seq_file.write(dna_seqs_6)
+        seq_file.close()
+        
         self._files_to_remove =\
-         [self.tmp_seq_filepath1, self.tmp_seq_filepath2, self.tmp_seq_filepath3]
+         [self.tmp_seq_filepath1, self.tmp_seq_filepath2, 
+          self.tmp_seq_filepath3, self.tmp_seq_filepath4]
         
     def tearDown(self):
         remove_files(self._files_to_remove)
@@ -584,6 +614,31 @@ class UclustOtuPickerTests(TestCase):
         seq_file.close()
         return fp
         
+    def test_toggle_collapse_identical_sequences(self):
+        """UclustOtuPicker: toggle prefilter identical seqs doesn't affect clusters
+        """
+        
+        # generate result including prefilter
+        app_w_collapse_identical =\
+          UclustOtuPicker(params={'Similarity':0.90,
+                                  'save_uc_files':True,
+                                  'prefilter_identical_sequences':True})
+        result_w_collapse_identical = \
+         app_w_collapse_identical(self.tmp_seq_filepath4).values()
+        result_w_collapse_identical.sort()
+        
+        # generate result excluding prefilter
+        app_wo_collapse_identical =\
+          UclustOtuPicker(params={'Similarity':0.90,
+                                  'save_uc_files':False,
+                                  'prefilter_identical_sequences':False})
+        result_wo_collapse_identical = \
+         app_wo_collapse_identical(self.tmp_seq_filepath4).values()
+        result_wo_collapse_identical.sort()
+        
+        self.assertEqual(result_w_collapse_identical,
+                         result_wo_collapse_identical)
+        
     def test_toggle_suppress_sort(self):
         """UclustOtuPicker: togging suppress sort functions as expected
         """
@@ -614,7 +669,7 @@ class UclustOtuPickerTests(TestCase):
                                       'presort_by_abundance':False,
                                       'save_uc_files':False})
         obs = app(seqs_fp)
-        exp = {0:['s1','s2','s3','s4','s5','s6']}
+        exp = {0:['s1','s4','s6','s2','s3','s5']}
         self.assertEqual(obs,exp)
         
     def test_abundance_sort(self):
@@ -680,12 +735,12 @@ class UclustOtuPickerTests(TestCase):
         # just checks that we have the expected group of each
         self.assertEqual(obs_otu_ids, exp_otu_ids)
         self.assertEqual(obs_clusters, exp_clusters)
-        
+
+
     def test_call_default_params_save_uc_file(self):
         """ returns expected clusters default params, writes correct .uc file"""
 
         # adapted from test_app.test_cd_hit.test_cdhit_clusters_from_seqs
-        
         exp_otu_ids = range(10)
         exp_clusters = [['uclust_test_seqs_0'],
                         ['uclust_test_seqs_1'],
@@ -702,7 +757,7 @@ class UclustOtuPickerTests(TestCase):
 
         
         app = UclustOtuPicker(params={'save_uc_files':True,
-                                      'output_dir':'/tmp/'})
+                                      'output_dir':self.temp_dir})
         obs = app(self.tmp_seq_filepath1)
         
 
@@ -943,6 +998,7 @@ class UclustOtuPickerTests(TestCase):
          "stable_sort:True",
          "output_dir:.",
          "save_uc_files:False",
+         "prefilter_identical_sequences:True",
          "Result path: %s" % tmp_result_filepath]
         # compare data in log file to fake expected log file
         # NOTE: Since app.params is a dict, the order of lines is not
@@ -984,6 +1040,7 @@ class UclustReferenceOtuPickerTests(TestCase):
     
     def setUp(self):
         """ """
+        self.temp_dir = load_qiime_config()['temp_dir']
         self.tmp_seq_filepath1 = get_tmp_filename(
          prefix='UclustReferenceOtuPickerTest_',
          suffix='.fasta')
@@ -1049,7 +1106,7 @@ class UclustReferenceOtuPickerTests(TestCase):
                                       'presort_by_abundance':False,
                                       'save_uc_files':False})
         obs = app(seqs_fp,ref_seqs_fp)
-        exp = {'r1':['s1','s2','s3','s4','s5','s6']}
+        exp = {'r1':['s1','s4','s6','s2','s3','s5']}
         self.assertEqual(obs,exp)
         
     def test_abundance_sort(self):
@@ -1088,7 +1145,6 @@ class UclustReferenceOtuPickerTests(TestCase):
         obs = app(seqs_fp,ref_seqs_fp)
         exp = {'r1':['s1','s4','s6','s3','s5','s2']}
         self.assertEqual(obs,exp)
-
         
     def test_toggle_suppress_new_clusters(self):
         """UclustReferenceOtuPicker: toggle suppress new clusters 
@@ -1122,10 +1178,102 @@ class UclustReferenceOtuPickerTests(TestCase):
         exp = {'r1':['s1','s2'],'new_42':['s3']}
         self.assertEqual(obs,exp)
         
-
         
-
+    def test_toggle_collapse_identical_sequences_prefilter_w_new_clusters(self):
+        """UclustReferenceOtuPicker: ident. seqs prefilter fns w new clusters
+        """
+        # s4 == s2 and s3 == s5
+        seqs = [('s1 a','ACCTTGTTACTTT'),
+                ('s2 bb','ACCTAGTTACTTT'),
+                ('s4 bb','ACCTAGTTACTTT'),
+                ('s3 c  c','TTGCGTAACGTTTGAC'),
+                ('s5 c  c','TTGCGTAACGTTTGAC')]
+        ref_seqs = [
+                ('r1 d','ACCTCGTTACTTT')]
+        exp = {'r1':['s2','s4','s1'],
+               'new_42':['s3','s5']}
+               
+        # add seq that clusters independently
+        uc = UclustReferenceOtuPicker({'Similarity':0.80,
+                                       'new_cluster_identifier':'new_',
+                                       'next_new_cluster_number':42,
+                                       'suppress_new_clusters':False,
+                                       'save_uc_files':False,
+                                       'prefilter_identical_sequences':False})
+        obs_no_prefilter = uc(self.seqs_to_temp_fasta(seqs),
+                 self.seqs_to_temp_fasta(ref_seqs),HALT_EXEC=False)
+        self.assertEqual(obs_no_prefilter,exp)
         
+        uc = UclustReferenceOtuPicker({'Similarity':0.80,
+                                       'new_cluster_identifier':'new_',
+                                       'next_new_cluster_number':42,
+                                       'suppress_new_clusters':False,
+                                       'save_uc_files':False,
+                                       'prefilter_identical_sequences':True})
+        obs_prefilter = uc(self.seqs_to_temp_fasta(seqs),
+                 self.seqs_to_temp_fasta(ref_seqs),HALT_EXEC=False)
+        self.assertEqual(obs_prefilter,exp)
+        
+        # a little paranoia never hurt anyone
+        self.assertEqual(obs_prefilter,obs_no_prefilter)
+
+    def test_toggle_collapse_identical_sequences_prefilter_wo_new_clusters(self):
+        """UclustReferenceOtuPicker: ident. seqs prefilter fns wo new clusters
+        """
+        # s4 == s2 and s3 == s5
+        seqs = [('s1 a','ACCTTGTTACTTT'),
+                ('s2 bb','ACCTAGTTACTTT'),
+                ('s4 bb','ACCTAGTTACTTT'),
+                ('s3 c  c','TTGCGTAACGTTTGAC'),
+                ('s5 c  c','TTGCGTAACGTTTGAC')]
+        ref_seqs = [
+                ('r1 d','ACCTCGTTACTTT')]
+        
+        exp = {'r1':['s2','s4','s1']}
+        
+        # add seq that clusters independently
+        uc = UclustReferenceOtuPicker({'Similarity':0.80,
+                                       'new_cluster_identifier':'new_',
+                                       'next_new_cluster_number':42,
+                                       'suppress_new_clusters':True,
+                                       'save_uc_files':False,
+                                       'prefilter_identical_sequences':False})
+        fail_path_no_prefilter = get_tmp_filename(
+         prefix='UclustRefOtuPickerFailures',suffix='.txt')
+        self._files_to_remove.append(fail_path_no_prefilter)
+        obs_no_prefilter = uc(self.seqs_to_temp_fasta(seqs),
+                              self.seqs_to_temp_fasta(ref_seqs),
+                              failure_path=fail_path_no_prefilter,
+                              HALT_EXEC=False)
+        self.assertEqual(obs_no_prefilter,exp)
+        self.assertEqual(open(fail_path_no_prefilter).read(),
+                         "s3\ns5")
+        
+        uc = UclustReferenceOtuPicker({'Similarity':0.80,
+                                       'new_cluster_identifier':'new_',
+                                       'next_new_cluster_number':42,
+                                       'suppress_new_clusters':True,
+                                       'save_uc_files':False,
+                                       'prefilter_identical_sequences':True})
+        fail_path_prefilter = get_tmp_filename(
+         prefix='UclustRefOtuPickerFailures',suffix='.txt')
+        self._files_to_remove.append(fail_path_prefilter)
+        obs_prefilter = uc(self.seqs_to_temp_fasta(seqs),
+                           self.seqs_to_temp_fasta(ref_seqs),
+                           failure_path=fail_path_prefilter,
+                           HALT_EXEC=False)
+        self.assertEqual(obs_prefilter,exp)
+        self.assertEqual(open(fail_path_prefilter).read(),
+                         "s3\ns5")
+        
+        # a little paranoia never hurt anyone
+        self.assertEqual(obs_prefilter,obs_no_prefilter)
+        self.assertEqual(open(fail_path_prefilter).read(),
+                         open(fail_path_no_prefilter).read())
+
+
+
+
     def test_varied_similarity(self):
         """UclustReferenceOtuPicker: varying similarity affects clustering
         """
@@ -1253,6 +1401,7 @@ class UclustReferenceOtuPickerTests(TestCase):
          "presort_by_abundance:True",
          'save_uc_files:False',
          'output_dir:.',
+         'prefilter_identical_sequences:True',
          "Result path: %s" % tmp_result_filepath]
         # compare data in log file to fake expected log file
         # NOTE: Since app.params is a dict, the order of lines is not
@@ -1309,7 +1458,7 @@ class UclustReferenceOtuPickerTests(TestCase):
         """UclustReferenceOtuPicker: default parameters, saves uc file
         """
         uc = UclustReferenceOtuPicker({'save_uc_files':True,
-                                       'output_dir':'/tmp/'})
+                                       'output_dir':self.temp_dir})
                                        
         obs = uc(self.tmp_seq_filepath1,self.temp_ref_filepath1)
         exp = {'ref1':['uclust_test_seqs_0'],
@@ -1967,6 +2116,27 @@ ACGGTGGCTACAAGACGTCCCATCCAACGGGTTGGATACTTAAGGCACATCACGTCAGTTTTGTGTCAGAGCT
 AGCTCTGACACAAAACTGACGTGATGTGCCTTAAGTATCCAACCCGTTGGATGGGACGTCTTGTAGCCACCGT
 """
 
+dna_seqs_6 = """>uclust_test_seqs_0 some comment0
+AACCCCCACGGTGGATGCCACACGCCCCATACAAAGGGTAGGATGCTTAAGACACATCGCGTCAGGTTTGTGTCAGGCCT
+>uclust_test_seqs_1 some comment1
+AACCCCCACGGTGGATGCCACACGCCCCATACAAAGGGTAGGATGCTTAAGACACATCGCGTCAGGTTTGTGTCAGGCCT
+>uclust_test_seqs_2 some comment2
+CCCCCACGGTGGCAGCAACACGTCACATACAACGGGTTGGATTCTAAAGACAAACCGCGTCAAAGTTGTGTCAGAACT
+>uclust_test_seqs_3 some comment3
+CCCCACGGTAGCTGCAACACGTCCCATACCACGGGTAGGATGCTAAAGACACATCGGGTCTGTTTTGTGTCAGGGCT
+>uclust_test_seqs_4 some comment4
+GCCACGGTGGGTACAACACGTCCACTACATCGGCTTGGAAGGTAAAGACACGTCGCGTCAGTATTGCGTCAGGGCT
+>uclust_test_seqs_5 some comment4_again
+AACCCCCACGGTGGATGCCACACGCCCCATACAAAGGGTAGGATGCTTAAGACACATCGCGTCAGGTTTGTGTCAGGCCT
+>uclust_test_seqs_6 some comment6
+CGCGGTGGCTGCAAGACGTCCCATACAACGGGTTGGATGCTTAAGACACATCGCAACAGTTTTGAGTCAGGGCT
+>uclust_test_seqs_7 some comment7
+AACCCCCACGGTGGATGCCACACGCCCCATACCAAGGGTAGGATGCTTAAGACACATCGCGTCAGGTTTGTGTCAGGCCT
+>uclust_test_seqs_8 some comment8
+CGGTGGCTGCAACACGTGGCATACAACGGGTTGGATGCTTAAGACACATCGCCTCAGTTTTGTGTCAGGGCT
+>uclust_test_seqs_9 some comment9
+GGTGGCTGAAACACATCCCATACAACGGGTTGGATGCTTAAGACACATCGCATCAGTTTTATGTCAGGGGA"""
+
 dna_seqs_4_result_prefilter =\
  {0: ['uclust_test_seqs_0','uclust_test_seqs_1','uclust_test_seqs_2']}
  
@@ -1977,26 +2147,26 @@ expected_uc_output =\
           '# For C and D types, PctId is average id with seed.',
           '# QueryStart and SeedStart are zero-based relative to start of sequence.',
           '# If minus strand, SeedStart is relative to reverse-complemented seed.',
-          'S\t0\t71\t*\t*\t*\t*\t*\tuclust_test_seqs_9 some comment9\t*',
-          'S\t1\t76\t*\t*\t*\t*\t*\tuclust_test_seqs_4 some comment4\t*',
-          'S\t2\t72\t*\t*\t*\t*\t*\tuclust_test_seqs_8 some comment8\t*',
-          'S\t3\t74\t*\t*\t*\t*\t*\tuclust_test_seqs_6 some comment6\t*',
-          'S\t4\t75\t*\t*\t*\t*\t*\tuclust_test_seqs_5 some comment4_again\t*',
-          'S\t5\t78\t*\t*\t*\t*\t*\tuclust_test_seqs_2 some comment2\t*',
-          'S\t6\t77\t*\t*\t*\t*\t*\tuclust_test_seqs_3 some comment3\t*',
-          'S\t7\t73\t*\t*\t*\t*\t*\tuclust_test_seqs_7 some comment7\t*',
-          'S\t8\t79\t*\t*\t*\t*\t*\tuclust_test_seqs_1 some comment1\t*',
-          'S\t9\t80\t*\t*\t*\t*\t*\tuclust_test_seqs_0 some comment0\t*',
-          'C\t0\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_9 some comment9\t*',
-          'C\t1\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_4 some comment4\t*',
-          'C\t2\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_8 some comment8\t*',
-          'C\t3\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_6 some comment6\t*',
-          'C\t4\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_5 some comment4_again\t*',
-          'C\t5\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_2 some comment2\t*',
-          'C\t6\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_3 some comment3\t*',
-          'C\t7\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_7 some comment7\t*',
-          'C\t8\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_1 some comment1\t*',
-          'C\t9\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_0 some comment0\t*']
+          'S\t0\t71\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_9\t*',
+          'S\t1\t76\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_4\t*',
+          'S\t2\t72\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_8\t*',
+          'S\t3\t74\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_6\t*',
+          'S\t4\t75\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_5\t*',
+          'S\t5\t78\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_2\t*',
+          'S\t6\t77\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_3\t*',
+          'S\t7\t73\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_7\t*',
+          'S\t8\t79\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_1\t*',
+          'S\t9\t80\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_0\t*',
+          'C\t0\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_9\t*',
+          'C\t1\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_4\t*',
+          'C\t2\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_8\t*',
+          'C\t3\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_6\t*',
+          'C\t4\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_5\t*',
+          'C\t5\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_2\t*',
+          'C\t6\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_3\t*',
+          'C\t7\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_7\t*',
+          'C\t8\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_1\t*',
+          'C\t9\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_0\t*']
  
 expected_ref_uc_file =\
     ['# Tab-separated fields:',
@@ -2004,29 +2174,30 @@ expected_ref_uc_file =\
     '# Record types (field 1): L=LibSeed, S=NewSeed, H=Hit, R=Reject, D=LibCluster, C=NewCluster, N=NoHit', '# For C and D types, PctId is average id with seed.',
     '# QueryStart and SeedStart are zero-based relative to start of sequence.',
     '# If minus strand, SeedStart is relative to reverse-complemented seed.',
-    'S\t4\t71\t*\t*\t*\t*\t*\tuclust_test_seqs_9 some comment\t*',
+    'S\t4\t71\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_9\t*',
     'L\t1\t91\t*\t*\t*\t*\t*\tref2 15 random bases prepended to uclust_test_seqs_1 and one mismatch\t*',
-    'H\t1\t76\t98.7\t+\t0\t0\t15I76M\tuclust_test_seqs_1 some comment bbb\tref2 15 random bases prepended to uclust_test_seqs_1 and one mismatch',
-    'S\t5\t72\t*\t*\t*\t*\t*\tuclust_test_seqs_8 some comment8\t*',
-    'S\t6\t74\t*\t*\t*\t*\t*\tuclust_test_seqs_6 some comment6\t*',
-    'S\t7\t75\t*\t*\t*\t*\t*\tuclust_test_seqs_5 some comment\t*',
+    'H\t1\t76\t98.7\t+\t0\t0\t15I76M\tQiimeExactMatch.uclust_test_seqs_1\tref2 15 random bases prepended to uclust_test_seqs_1 and one mismatch',
+    'S\t5\t72\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_8\t*',
+    'S\t6\t74\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_6\t*',
+    'S\t7\t75\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_5\t*',
     'L\t2\t93\t*\t*\t*\t*\t*\tref3 5 random bases prepended and 10 random bases appended to uclust_test_seqs_2\t*',
-    'H\t2\t78\t100.0\t+\t0\t0\t5I78M10I\tuclust_test_seqs_2 some comment vv\tref3 5 random bases prepended and 10 random bases appended to uclust_test_seqs_2',
+    'H\t2\t78\t100.0\t+\t0\t0\t5I78M10I\tQiimeExactMatch.uclust_test_seqs_2\tref3 5 random bases prepended and 10 random bases appended to uclust_test_seqs_2',
     'L\t3\t77\t*\t*\t*\t*\t*\tref4 exact match to uclust_test_seqs_3\t*',
-    'H\t3\t77\t100.0\t+\t0\t0\t77M\tuclust_test_seqs_3 some comment\tref4 exact match to uclust_test_seqs_3',
+    'H\t3\t77\t100.0\t+\t0\t0\t77M\tQiimeExactMatch.uclust_test_seqs_3\tref4 exact match to uclust_test_seqs_3',
     'L\t0\t98\t*\t*\t*\t*\t*\tref1 25 random bases appended to uclust_test_seqs_0 and one mismatch\t*',
-    'H\t0\t73\t98.6\t+\t0\t0\t73M25I\tuclust_test_seqs_0 some comment aaa\tref1 25 random bases appended to uclust_test_seqs_0 and one mismatch',
-    'S\t8\t79\t*\t*\t*\t*\t*\tuclust_test_seqs_4 some comment\t*',
-    'S\t9\t80\t*\t*\t*\t*\t*\tuclust_test_seqs_7 some comment\t*',
+    'H\t0\t73\t98.6\t+\t0\t0\t73M25I\tQiimeExactMatch.uclust_test_seqs_0\tref1 25 random bases appended to uclust_test_seqs_0 and one mismatch',
+    'S\t8\t79\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_4\t*',
+    'S\t9\t80\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_7\t*',
     'D\t0\t2\t*\t*\t*\t*\t98.6\tref1 25 random bases appended to uclust_test_seqs_0 and one mismatch\t*',
     'D\t1\t2\t*\t*\t*\t*\t98.7\tref2 15 random bases prepended to uclust_test_seqs_1 and one mismatch\t*',
     'D\t2\t2\t*\t*\t*\t*\t100.0\tref3 5 random bases prepended and 10 random bases appended to uclust_test_seqs_2\t*',
-    'D\t3\t2\t*\t*\t*\t*\t100.0\tref4 exact match to uclust_test_seqs_3\t*', 'C\t4\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_9 some comment\t*',
-    'C\t5\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_8 some comment8\t*',
-    'C\t6\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_6 some comment6\t*',
-    'C\t7\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_5 some comment\t*',
-    'C\t8\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_4 some comment\t*',
-    'C\t9\t1\t*\t*\t*\t*\t*\tuclust_test_seqs_7 some comment\t*']
+    'D\t3\t2\t*\t*\t*\t*\t100.0\tref4 exact match to uclust_test_seqs_3\t*', 
+    'C\t4\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_9\t*',
+    'C\t5\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_8\t*',
+    'C\t6\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_6\t*',
+    'C\t7\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_5\t*',
+    'C\t8\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_4\t*',
+    'C\t9\t1\t*\t*\t*\t*\t*\tQiimeExactMatch.uclust_test_seqs_7\t*']
 
 #run unit tests if run from command-line
 if __name__ == '__main__':
