@@ -3,11 +3,11 @@
 """Tests of code for representative set picking"""
 
 __author__ = "Rob Knight"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 #remember to add yourself if you make changes
 __credits__ = ["Rob Knight", "Kyle Bittinger", "Greg Caporaso"] 
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Daniel McDonald"
 __email__ = "wasade@gmail.com"
 __status__ = "Release"
@@ -16,7 +16,7 @@ from os import remove
 from cogent import LoadSeqs
 from cogent.util.misc import remove_files
 from cogent.util.unit_test import TestCase, main
-from cogent.app.util import get_tmp_filename
+from qiime.util import get_tmp_filename
 from qiime.pick_rep_set import (RepSetPicker, GenericRepSetPicker, first_id,
     first, random_id, longest_id, unique_id_map, label_to_name, 
     make_most_abundant, MinimalFastaParser, ReferenceRepSetPicker)
