@@ -3,11 +3,11 @@
 """Tests of code for assigning taxonomy"""
 
 __author__ = "Dan Knights"
-__copyright__ = "Copyright 2010, The QIIME Project" 
+__copyright__ = "Copyright 2011, The QIIME Project" 
 #remember to add yourself if you make changes
 __credits__ = ["Dan Knights"] 
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Dan Knights"
 __email__ = "daniel.knights@colorado.edu"
 __status__ = "Release"
@@ -18,7 +18,9 @@ from shutil import rmtree
 from os.path import join, exists
 from tempfile import NamedTemporaryFile, mkdtemp
 from cogent.util.unit_test import TestCase, main
-from cogent.app.util import get_tmp_filename, ApplicationError
+from cogent.app.util import ApplicationError
+from qiime.util import get_tmp_filename
+
 from cogent.util.misc import remove_files
 from qiime.supervised_learning import RSupervisedLearner,\
     RSupervisedLearnerFilter, R_format_table
