@@ -3,15 +3,16 @@
 
 from os import rmdir
 from os.path import split, abspath, dirname, exists, join
+from glob import glob
 from cogent import Sequence
 from cogent.util.unit_test import TestCase, main
 from cogent.parse.fasta import MinimalFastaParser
-from cogent.app.util import get_tmp_filename
+from qiime.util import get_tmp_filename
 from cogent.util.misc import remove_files
 from cogent.cluster.procrustes import procrustes
 from cogent.app.formatdb import build_blast_db_from_fasta_file
 from cogent.util.misc import get_random_directory_name, remove_files
-
+from StringIO import StringIO
 from qiime.parse import fields_to_dict, parse_otu_table, parse_mapping_file
 from qiime.util import (make_safe_f, FunctionWithParams, qiime_blast_seqs,
     extract_seqs_by_sample_id, get_qiime_project_dir, matrix_stats,
@@ -20,20 +21,26 @@ from qiime.util import (make_safe_f, FunctionWithParams, qiime_blast_seqs,
     summarize_pcoas, _compute_jn_pcoa_avg_ranges, _flip_vectors, IQR,
     idealfourths, isarray, matrix_IQR, degap_fasta_aln,
     write_degapped_fasta_to_file, compare_otu_maps, get_diff_for_otu_maps,
-    merge_n_otu_tables, convert_otu_table_relative,
-    split_fasta_on_sample_ids, split_fasta_on_sample_ids_to_dict)
+    merge_n_otu_tables, convert_otu_table_relative, write_seqs_to_fasta,
+    split_fasta_on_sample_ids, split_fasta_on_sample_ids_to_dict,
+    split_fasta_on_sample_ids_to_files, median_absolute_deviation,
+    guess_even_sampling_depth, compute_days_since_epoch,
+    get_interesting_mapping_fields,inflate_denoiser_output,
+    flowgram_id_to_seq_id_map, count_seqs, count_seqs_from_file,
+    count_seqs_in_filepaths,get_split_libraries_fastq_params_and_file_types,
+    iseq_to_qseq_fields,get_top_fastq_two_lines,make_compatible_distance_matrices)
 
 import numpy
 from numpy import array, asarray
 
 
 __author__ = "Greg Caporaso"
-__copyright__ = "Copyright 2010, The QIIME Project"
+__copyright__ = "Copyright 2011, The QIIME Project"
 #remember to add yourself if you make changes
 __credits__ = ["Rob Knight", "Daniel McDonald","Greg Caporaso", 
                "Justin Kuczynski", "Jens Reeder", "Catherine Lozupone"] 
 __license__ = "GPL"
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
 __status__ = "Release"
@@ -49,15 +56,63 @@ class TopLevelTests(TestCase):
         self.otu_table_f2_no_tax = otu_table_fake2_no_tax.split('\n')
         self.otu_table_f3_no_tax = otu_table_fake3_no_tax.split('\n')
         self.fasta1 = fasta1.split('\n')
+        self.fasta2 = fasta2.split('\n')
         self.mapping_f1 = mapping_f1.split('\n')
+        self.mapping_f2 = mapping_f2.split('\n')
+        self.mapping_f3 = mapping_f3.split('\n')
         self.dirs_to_remove = []
         self.files_to_remove = []
+        self.centroid_seqs1 = centroid_seqs1.split('\n')
+        self.singleton_seqs1 = singleton_seqs1.split('\n')
+        self.denoiser_mapping1 = denoiser_mapping1.split('\n')
+        self.raw_seqs1 = raw_seqs1.split('\n')
 
-    def tearDown(self):
+    def tearDown(self):    
+        remove_files(self.files_to_remove)
         for dir in  self.dirs_to_remove:
             if exists(dir):
                 rmdir(dir)
-        remove_files(self.files_to_remove)
+                
+    def test_median_absolute_deviation(self):
+        """ median_absolute_deviation returns MAD and median """
+        data = [0,0,0,0,0,0]
+        expected = (0,0)
+        self.assertEqual(median_absolute_deviation(data),expected)
+        data = [1,1,1,1,1,1]
+        expected = (0,1)
+        self.assertEqual(median_absolute_deviation(data),expected)
+        data = [6,1,2,9,4,1,2]
+        expected = (1,2)
+        self.assertEqual(median_absolute_deviation(data),expected)
+        data = [-6,-1,-2,-9,-4,-1,-2]
+        expected = (1,-2)
+        self.assertEqual(median_absolute_deviation(data),expected)
+        
+    def test_guess_even_sampling_depth(self):
+        """ guess_even_sampling_depth functions as expected """
+        data = [6,1,2,9,4,1,2]
+        expected = 1 # MAD = 2.25; med - MAD = -0.25
+        self.assertEqual(guess_even_sampling_depth(data),expected)
+    
+    def test_write_seqs_to_fasta(self):
+        """ write_seqs_to_fasta functions as expected """
+        output_fp = get_tmp_filename(
+                     prefix="qiime_util_write_seqs_to_fasta_test",
+                     suffix='.fasta')
+        self.files_to_remove.append(output_fp)
+        seqs = [('s1','ACCGGTTGG'),('s2','CCTTGG'),('S4 some comment string','A')]
+        exp = ">s1\nACCGGTTGG\n>s2\nCCTTGG\n>S4 some comment string\nA\n"
+        # works in write mode
+        write_seqs_to_fasta(output_fp,seqs,'w')
+        self.assertEqual(open(output_fp).read(),exp)
+        # calling again in write mode overwrites original file
+        write_seqs_to_fasta(output_fp,seqs,'w')
+        self.assertEqual(open(output_fp).read(),exp)
+        # works in append mode
+        exp2 = exp + exp
+        write_seqs_to_fasta(output_fp,seqs,'a')
+        self.assertEqual(open(output_fp).read(),exp2)
+        
     
     def test_split_fasta_on_sample_ids(self):
         """ split_fasta_on_sample_ids functions as expected 
@@ -80,6 +135,29 @@ class TopLevelTests(TestCase):
                     's2_a':[('s2_a_50','GGGCCC')],
                     's3':[('s3_25','AAACCC')]}
         self.assertEqual(actual,expected)
+
+    def test_split_fasta_on_sample_ids_to_files(self):
+        """ split_fasta_on_sample_ids_to_files functions as expected 
+        """
+        temp_output_dir = get_random_directory_name(output_dir='/tmp/')
+        self.dirs_to_remove.append(temp_output_dir)
+        
+        split_fasta_on_sample_ids_to_files(
+         MinimalFastaParser(self.fasta2),
+         output_dir=temp_output_dir,
+         per_sample_buffer_size=2)
+        self.files_to_remove.extend(glob('%s/*fasta' % temp_output_dir))
+        
+        # confirm that all files are as expected
+        self.assertEqual(open('%s/Samp1.fasta' % temp_output_dir).read(),
+            ">Samp1_42\nACCGGTT\n>Samp1_43 some comme_nt\nAACCG\n>Samp1_44\nA\n")
+        self.assertEqual(open('%s/s2_a.fasta' % temp_output_dir).read(),
+            ">s2_a_50\nGGGCCC\n")
+        self.assertEqual(open('%s/s3.fasta' % temp_output_dir).read(),
+            ">s3_25\nAAACCC\n")
+        # confirm number of files is as expected
+        self.assertEqual(len(glob('%s/*' % temp_output_dir)),3)
+
                 
     def test_convert_otu_table_relative(self):
         """should convert a parsed otu table into relative abundances"""
@@ -275,7 +353,29 @@ class TopLevelTests(TestCase):
         self.assertEqual(actual[1],exp_otu_ids)
         self.assertEqual(actual[2],exp_otu_table)
         self.assertEqual(actual[3],exp_lineages)
-        
+    
+    def test_merge_otu_tables_passed_in_tables(self):
+        """merge_otu_tables functions with otu_table tuples"""
+        otu_table1 = parse_otu_table(self.otu_table_f1)
+        otu_table2 = parse_otu_table(self.otu_table_f2)
+        exp_sample_ids = ['S1','S2','S3','S4','S5']
+        exp_otu_ids = ['0','1','2','3','4','6']
+        exp_otu_table = array([[1,0,1,0,1],\
+                           [1,0,0,0,0],\
+                           [4,0,1,0,1],\
+                           [0,0,2,0,1],\
+                           [0,0,1,0,9],\
+                           [0,0,1,25,42]])
+        exp_lineages = [['Root','Bacteria'],\
+                    ['Root','Bacteria','Verrucomicrobia'],
+                    ['Root','Bacteria'],\
+                    ['Root','Bacteria','Acidobacteria'],\
+                    ['Root','Bacteria','Bacteroidetes'],\
+                    ['Root','Archaea']]
+        exp = (exp_sample_ids,exp_otu_ids,exp_otu_table,exp_lineages)
+        obs = merge_otu_tables(otu_table1,otu_table2)
+        self.assertEqual(obs,exp)
+
     def test_merge_n_otu_tables(self):
         """merge_n_otu_tables functions as expected"""
         otu_table_f1 = iter(self.otu_table_f1)
@@ -341,8 +441,317 @@ class TopLevelTests(TestCase):
         self.assertRaises(ValueError,
          merge_n_otu_tables,[otu_table_f1,otu_table_f2,otu_table_f3])
         
+    def test_compute_days_since_epoch(self):
+        """compute_days_since_epoch functions as expected """
+        self.assertEqual(compute_days_since_epoch(29,10,2002),11989)
+        # can pass keyword arguments
+        self.assertEqual(compute_days_since_epoch(year=2002,month=10,day=29),11989)
+        self.assertEqual(compute_days_since_epoch(day=27,month=3,year=2009),14330)
+        self.assertEqual(compute_days_since_epoch(day="27",month="3",year="2009"),14330)
+    
+    def test_get_interesting_mapping_fields(self):
+        """get_interesting_mapping_fields returns expected fields """
+        # all columns are completely unique
+        d = parse_mapping_file(self.mapping_f1)
+        actual = get_interesting_mapping_fields(d[0],d[1])
+        expected = []
+        self.assertEqual(actual,expected)
+        
+        # all columns are completely identical
+        d = parse_mapping_file(self.mapping_f2)
+        actual = get_interesting_mapping_fields(d[0],d[1])
+        expected = []
+        self.assertEqual(actual,expected)
+        
+        # some columns retained
+        d = parse_mapping_file(self.mapping_f3)
+        actual = get_interesting_mapping_fields(d[0],d[1])
+        expected = ['Something','days_since_epoch']
+        self.assertEqual(actual,expected)
+        
+    def test_inflate_denoiser_output(self):
+        """ inflate_denoiser_output expands denoiser results as expected """
+        actual = list(inflate_denoiser_output(
+         MinimalFastaParser(self.centroid_seqs1),
+         MinimalFastaParser(self.singleton_seqs1),
+         self.denoiser_mapping1,
+         MinimalFastaParser(self.raw_seqs1)))
+        expected = [("S1_0 FXX111 some comments","TTTT"),
+                    ("S1_2 FXX113 some other comments","TTTT"),
+                    ("S3_7 FXX117","TTTT"),
+                    ("S2_1 FXX112 some comments","TATT"),
+                    ("S3_5 FXX114","TATT"),
+                    ("S3_6 FXX115","TTGA"),
+                    ("S3_6 FXX116","TAGA")]
+        self.assertEqual(actual,expected)
+    
+    def test_flowgram_id_to_seq_id_map(self):
+        """ flowgram_id_to_seq_id_map functions as expected """
+        actual = flowgram_id_to_seq_id_map(MinimalFastaParser(self.raw_seqs1))
+        expected = {'FXX111':'S1_0 FXX111 some comments',
+                    'FXX112':'S2_1 FXX112 some comments',
+                    'FXX113':'S1_2 FXX113 some other comments',
+                    'FXX114':'S3_5 FXX114',
+                    'FXX115':'S3_6 FXX115',
+                    'FXX116':'S3_6 FXX116',
+                    'FXX117':'S3_7 FXX117'}
+        self.assertEqual(actual,expected)
 
 
+    def test_count_seqs_from_file(self):
+        """ count_seqs: functions as expected with varied data
+        """
+        f1 = ['>seq1','AACCTT','ACTGGT',
+              '>seq2','CCAATT',
+              '>seq3','CCC---GG']
+        f2 = ['> s42','ABCDEFG',
+              '>s33','A',
+              '> 4>','AA>',
+              '>blah','AA']
+        self.assertFloatEqual(count_seqs_from_file(f1),(3,8.666,2.4944),0.001)
+        self.assertFloatEqual(count_seqs_from_file(f2),(4,3.25,2.2776),0.001)
+        self.assertEqual(count_seqs_from_file([]),(0,None,None))
+        
+    def test_count_seqs(self):
+        """ count_seqs functions as expected with fake seq_counter
+        """
+        def seq_counter(filepath):
+            # Fake sequence counter to test count_seqs without
+            # having to write files to disk (note don't need to
+            # test actual sequence counters here as they're tested
+            # elsewhere)
+            if filepath.startswith('fake'):
+                raise IOError
+            else:
+                return len(filepath), 0, 0
+                
+        in_fps = ['1.fasta','fake1.fasta','fake.fasta','2.fa']
+        expected = [((7,0,0),'1.fasta'),
+                    ((4,0,0),'2.fa')],\
+                    11, ['fake1.fasta','fake.fasta']
+        self.assertEqual(count_seqs_in_filepaths(\
+         in_fps,seq_counter),expected)
+        
+        in_fps = ['fake1.fasta','fake.fasta']
+        expected = [], 0, ['fake1.fasta','fake.fasta']
+        self.assertEqual(count_seqs_in_filepaths(\
+         in_fps,seq_counter),expected)
+                
+        in_fps = ['1.fasta','2.fa','12.txt']
+        expected = [((7,0,0),'1.fasta'),
+                    ((4,0,0),'2.fa'),
+                    ((6,0,0),'12.txt')], 17, []
+        self.assertEqual(count_seqs_in_filepaths(\
+         in_fps,seq_counter),expected)
+
+    def test_get_split_libraries_fastq_params_and_file_types_reverse(self):
+        """get_split_libraries_fastq_params_and_file_types using reverse 
+           barcodes computes correct values"""
+    
+        temp_output_dir = get_random_directory_name(output_dir='/tmp/')
+        self.dirs_to_remove.append(temp_output_dir)
+
+        #generate the fastq mapping file
+        map_fpath=join(temp_output_dir,'map.txt')
+        map_fopen=open(map_fpath,'w')
+        map_fopen.write('\n'.join(fastq_mapping_rev))
+        map_fopen.close()
+        self.files_to_remove.append(map_fpath)
+
+        fastq_files=[]
+        #generate fastq seqs file
+        seq_fpath=join(temp_output_dir,'seqs.fastq')
+        seqs_fopen=open(seq_fpath,'w')
+        seqs_fopen.write('\n'.join(fastq_seqs))
+        seqs_fopen.close()
+
+        fastq_files.append(seq_fpath)
+        self.files_to_remove.append(seq_fpath)
+
+        #generate fastq seqs file
+        barcode_fpath=join(temp_output_dir,'barcodes.fastq')
+        barcode_fopen=open(barcode_fpath,'w')
+        barcode_fopen.write('\n'.join(fastq_barcodes))
+        barcode_fopen.close()
+
+        fastq_files.append(barcode_fpath)
+        self.files_to_remove.append(barcode_fpath)
+
+        exp='-i %s -b %s --rev_comp_barcode' % (seq_fpath,barcode_fpath)
+
+        obs=get_split_libraries_fastq_params_and_file_types(fastq_files,
+                                                           map_fpath)
+
+        self.assertEqual(obs,exp)
+
+    def test_get_split_libraries_fastq_params_and_file_types_forward(self):
+        """get_split_libraries_fastq_params_and_file_types using forward
+           barcodes computes correct values"""
+    
+        temp_output_dir = get_random_directory_name(output_dir='/tmp/')
+        self.dirs_to_remove.append(temp_output_dir)
+    
+        #generate the fastq mapping file
+        map_fpath=join(temp_output_dir,'map.txt')
+        map_fopen=open(map_fpath,'w')
+        map_fopen.write('\n'.join(fastq_mapping_fwd))
+        map_fopen.close()
+        self.files_to_remove.append(map_fpath)
+        
+        fastq_files=[]
+        #generate fastq seqs file
+        seq_fpath=join(temp_output_dir,'seqs.fastq')
+        seqs_fopen=open(seq_fpath,'w')
+        seqs_fopen.write('\n'.join(fastq_seqs))
+        seqs_fopen.close()
+        
+        fastq_files.append(seq_fpath)
+        self.files_to_remove.append(seq_fpath)
+        
+        #generate fastq seqs file
+        barcode_fpath=join(temp_output_dir,'barcodes.fastq')
+        barcode_fopen=open(barcode_fpath,'w')
+        barcode_fopen.write('\n'.join(fastq_barcodes))
+        barcode_fopen.close()
+        
+        fastq_files.append(barcode_fpath)
+        self.files_to_remove.append(barcode_fpath)
+        
+        exp='-i %s -b %s ' % (seq_fpath,barcode_fpath)
+    
+        obs=get_split_libraries_fastq_params_and_file_types(fastq_files,
+                                                            map_fpath)
+    
+        self.assertEqual(obs,exp)
+
+    def test_get_top_fastq_two_lines(self):
+        """ get_top_fastq_two_lines: this function gets the first 4 lines of 
+            the open fastq file
+        """
+        
+        temp_output_dir = get_random_directory_name(output_dir='/tmp/')
+        self.dirs_to_remove.append(temp_output_dir)
+
+        fastq_files=[]
+        #generate fastq seqs file
+        seq_fpath=join(temp_output_dir,'seqs.fastq')
+        seqs_fopen=open(seq_fpath,'w')
+        seqs_fopen.write('\n'.join(fastq_seqs))
+        seqs_fopen.close()
+
+        fastq_files.append(seq_fpath)
+        self.files_to_remove.append(seq_fpath)
+
+        #generate fastq seqs file
+        barcode_fpath=join(temp_output_dir,'barcodes.fastq')
+        barcode_fopen=open(barcode_fpath,'w')
+        barcode_fopen.write('\n'.join(fastq_barcodes))
+        barcode_fopen.close()
+
+        fastq_files.append(barcode_fpath)
+        self.files_to_remove.append(barcode_fpath)
+        exp=[('@HWUSI-EAS552R_0357:8:1:10040:6364#0/1\n', 'GACGAGTCAGTC\n', 
+              '+HWUSI-EAS552R_0357:8:1:10040:6364#0/1\n', 'hhhhhhhhhhhh\n'),
+             ('@HWUSI-EAS552R_0357:8:1:10040:6364#0/2\n', 
+              'TACAGGGGATGCAAGTGTTATCCGGAATTATTGGGCGTAAAGCGTCTGCAGGTTGCTCACTAAGTCTTTTGTTAAATCTTCGGGCTTAACCCGAAACCTGCAAAAGAAACTAGTGCTCTCGAGTATGGTAGAGGTAAAGGGAATTTCCAG\n', 
+              '+HWUSI-EAS552R_0357:8:1:10040:6364#0/2\n', 
+              'hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhfhhhhhhgghhhhhWhfcffehf]hhdhhhhhgcghhhchhhhfhcfhhgggdfhgdcffadccfdcccca]^b``ccfdd_caccWbb[b_dfdcdeaec`^`^_daba_b_WdY^`\n')]
+        
+        #iterate of dict and make sure the top 4 lines of each file are in the
+        #expected list
+        for i in fastq_files:
+            obs=get_top_fastq_two_lines(open(i))
+            self.assertTrue(obs in exp)
+            
+    def test_make_compatible_distance_matrices(self):
+        """make_compatible_distance_matrices: functions as expected"""
+        dm1 = (['A','B','C','D'],
+               array([[0.0,2.3,3.3,4.3],
+                      [2.9,0.0,5.3,6.3],
+                      [3.9,5.9,0.0,4.3],
+                      [4.9,8.9,9.9,0.0]]))
+        
+        dm2 = (['C','A','T'],
+               array([[10.0,12.3,13.3],
+                      [12.9,10.0,15.3],
+                      [13.9,15.9,10.0]]))
+        
+        expected_dm1 = (['A','C'],
+               array([[0.0,3.3],
+                      [3.9,0.0]]))
+        
+        expected_dm2 = (['A','C'],
+               array([[10.0,12.9],
+                      [12.3,10.0]]))
+        
+        actual_dm1, actual_dm2 = make_compatible_distance_matrices(dm1,dm2)
+        self.assertEqual(actual_dm1,expected_dm1)
+        self.assertEqual(actual_dm2,expected_dm2)
+
+    def test_make_compatible_distance_matrices_w_lookup(self):
+        """make_compatible_distance_matrices: functions as expected with lookup"""
+        dm1 = (['A','B','C','D'],
+               array([[0.0,2.3,3.3,4.3],
+                      [2.9,0.0,5.3,6.3],
+                      [3.9,5.9,0.0,4.3],
+                      [4.9,8.9,9.9,0.0]]))
+        
+        dm2 = (['C','A','T'],
+               array([[10.0,12.3,13.3], 
+                      [12.9,10.0,15.3], 
+                      [13.9,15.9,10.0]]))
+                      
+        lookup = {'C':'C','A':'A','B':'B','T':'B','D':'D'}
+        
+        expected_dm1 = (['A','B','C'],
+               array([[0.0,2.3,3.3],
+                      [2.9,0.0,5.3],
+                      [3.9,5.9,0.0]]))
+        
+        expected_dm2 = (['A','B','C'],
+               array([[10.0,15.3,12.9],
+                      [15.9,10.0,13.9],
+                      [12.3,13.3,10.0]]))
+        
+        actual_dm1, actual_dm2 = make_compatible_distance_matrices(dm1,dm2,lookup)
+        self.assertEqual(actual_dm1,expected_dm1)
+        self.assertEqual(actual_dm2,expected_dm2)
+        
+        lookup = {'C':'C','B':'B','T':'B','D':'D'}
+        self.assertRaises(KeyError,
+         make_compatible_distance_matrices,dm1,dm2,lookup)
+
+
+raw_seqs1 = """>S1_0 FXX111 some comments
+TTTT
+>S2_1 FXX112 some comments
+TATT
+>S1_2 FXX113 some other comments
+GGGG
+>S3_5 FXX114
+GGGA
+>S3_6 FXX115
+TTGA
+>S3_6 FXX116
+TAGA
+>S3_7 FXX117
+TAGT"""
+
+centroid_seqs1 = """>FXX111 | cluster size: 3
+TTTT
+>FXX112 | cluster size: 2
+TATT"""
+
+singleton_seqs1 = """>FXX115
+TTGA
+>FXX116
+TAGA"""
+        
+denoiser_mapping1 = """FXX111:\tFXX113\tFXX117
+FXX115:
+FXX112:\tFXX114
+FXX116:"""
+
 
 otu_table_fake1 = """#Full OTU Counts
 #OTU ID	S1	S2	Consensus Lineage
@@ -749,6 +1158,33 @@ AAAAAAA
         self.assertFloatEqual(compare_otu_maps(otu_map1, otu_map4), 0.33333333333)
         self.assertFloatEqual(compare_otu_maps(otu_map3, otu_map4), 0.33333333333)
         self.assertFloatEqual(compare_otu_maps(otu_map1, otu_map5), 1)
+
+    def test_iseq_to_qseq_fields(self):
+        """iseq_to_qseq_fields functions as expected"""
+        i = "HWI-ST753_50:6:1101:15435:9071#0/1:ACCAGACGATGCTACGGAGGGAGCTAGCGTTGTTCGGAATTACTGGGCGTAAAGCGCACGTAGGCGGCTTTGTAAGTTAGAGGTGAAAGCCTGGAGCTCAAC:gggggggfggdegggggggggggggggggggegggggggggegggggggeggcccccFUZSU_]]^^ggggggdggdgeeeccYacadcbeddceegggeeg"
+        # barcode in sequence, barcode length = 12
+        expected = (("HWI-ST753","50","6","1101","15435","9071","0","1"),
+                    "TACGGAGGGAGCTAGCGTTGTTCGGAATTACTGGGCGTAAAGCGCACGTAGGCGGCTTTGTAAGTTAGAGGTGAAAGCCTGGAGCTCAAC","gggggggggggggggggggegggggggggegggggggeggcccccFUZSU_]]^^ggggggdggdgeeeccYacadcbeddceegggeeg","ACCAGACGATGC","gggggggfggde")
+        self.assertEqual(iseq_to_qseq_fields(i,barcode_in_header=False,barcode_length=12),
+                         expected)
+        # barcode in sequence, barcode length = 6
+        expected = (("HWI-ST753","50","6","1101","15435","9071","0","1"),
+                    "CGATGCTACGGAGGGAGCTAGCGTTGTTCGGAATTACTGGGCGTAAAGCGCACGTAGGCGGCTTTGTAAGTTAGAGGTGAAAGCCTGGAGCTCAAC","gfggdegggggggggggggggggggegggggggggegggggggeggcccccFUZSU_]]^^ggggggdggdgeeeccYacadcbeddceegggeeg","ACCAGA","gggggg")
+        self.assertEqual(iseq_to_qseq_fields(i,barcode_in_header=False,barcode_length=6),
+                         expected)
+                         
+        # barcode in header, barcode length = 6
+        i = "HWI-6X_9267:1:1:4:1699#ACCACCC/1:TACGGAGGGTGCGAGCGTTAATCGCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCGAAAAAAAAAAAAAAAAAAAAAAA:abbbbbbbbbb`_`bbbbbb`bb^aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaDaabbBBBBBBBBBBBBBBBBBBB"
+        expected = (("HWI-6X","9267","1","1","4","1699","ACCACCC", "1"),
+         "TACGGAGGGTGCGAGCGTTAATCGCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCGAAAAAAAAAAAAAAAAAAAAAAA","abbbbbbbbbb`_`bbbbbb`bb^aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaDaabbBBBBBBBBBBBBBBBBBBB","ACCACC","bbbbbb")
+        self.assertEqual(iseq_to_qseq_fields(i,barcode_in_header=True,barcode_length=6),
+                         expected)
+        # barcode in header, barcode length = 3
+        expected = (("HWI-6X","9267","1","1","4","1699","ACCACCC", "1"),
+         "TACGGAGGGTGCGAGCGTTAATCGCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCGAAAAAAAAAAAAAAAAAAAAAAA","abbbbbbbbbb`_`bbbbbb`bb^aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaDaabbBBBBBBBBBBBBBBBBBBB","ACC","bbb")
+        self.assertEqual(iseq_to_qseq_fields(i,barcode_in_header=True,barcode_length=3),
+                         expected)
+
  
 otu_map1 = fields_to_dict("""1:\ta\tb\tc
 2:\td
@@ -822,7 +1258,55 @@ AACCG
 >s3_25
 AAACCC"""
 
+fasta2 = """>Samp1_42
+ACCGGTT
+>s2_a_50
+GGGCCC
+>Samp1_43 some comme_nt
+AACCG
+>s3_25
+AAACCC
+>Samp1_44
+A"""
 
+mapping_f2 = """#SampleID\tSomething\tdays_since_epoch
+Z1\thello\t23
+Z2\thello\t23
+A\thello\t23
+1\thello\t23
+NotInOtuTable\thello\t23"""
+
+mapping_f3 = """#SampleID\tSomething\tdays_since_epoch
+Z1\t42\t23
+Z2\t42\t10
+A\t4\t400000
+1\t4\t5.7
+NotInOtuTable\t9\t5.7"""
+
+fastq_barcodes=["@HWUSI-EAS552R_0357:8:1:10040:6364#0/1",
+"GACGAGTCAGTC",
+"+HWUSI-EAS552R_0357:8:1:10040:6364#0/1",
+"hhhhhhhhhhhh",
+"@HWUSI-EAS552R_0357:8:1:10184:6365#0/1",
+"GTCTGACAGTTG",
+"+HWUSI-EAS552R_0357:8:1:10184:6365#0/1",
+"hhhhhhhhhhhh"]
+fastq_seqs=["@HWUSI-EAS552R_0357:8:1:10040:6364#0/2",
+"TACAGGGGATGCAAGTGTTATCCGGAATTATTGGGCGTAAAGCGTCTGCAGGTTGCTCACTAAGTCTTTTGTTAAATCTTCGGGCTTAACCCGAAACCTGCAAAAGAAACTAGTGCTCTCGAGTATGGTAGAGGTAAAGGGAATTTCCAG",
+"+HWUSI-EAS552R_0357:8:1:10040:6364#0/2",
+"hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhfhhhhhhgghhhhhWhfcffehf]hhdhhhhhgcghhhchhhhfhcfhhgggdfhgdcffadccfdcccca]^b``ccfdd_caccWbb[b_dfdcdeaec`^`^_daba_b_WdY^`",
+"@HWUSI-EAS552R_0357:8:1:10184:6365#0/2",
+"TACGAAGGGGGCTAGCGTTGCTCGGAATCACTGGGCGTAAAGCGCACGTAGGCGGGCTCTTAAGTCGGAGGTGAAATCCCAAGGCTCAACCTTGGAACTGCCTTCGATACTGAGAGTCTTGAGTCCGGAAGAGGTAAGTGGAACTCCAAG",
+"+HWUSI-EAS552R_0357:8:1:10184:6365#0/2",
+"hfhhchhghhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhfghghhhggfhhfghfeghfggfdhdfdfffacbfcddgcfccddbcddbccada_aadWaaaacddccccdacaaa_acbc]c`aa[a\\a_a^V\T_^^^^X^R_BBBB"]
+
+fastq_mapping_rev=["#SampleID\tBarcodeSequence\tLinkerPrimerSequence\tDescription",
+"sample1\tGACTGACTCGTC\tCCGGACTACHVGGGTWTCTAAT\tsample1",
+"sample2\tCAACTGTCAGAC\tCCGGACTACHVGGGTWTCTAAT\tsample2"]
+
+fastq_mapping_fwd=["#SampleID\tBarcodeSequence\tLinkerPrimerSequence\tDescription",
+"sample1\tGACGAGTCAGTC\tCCGGACTACHVGGGTWTCTAAT\tsample1",
+"sample2\tGTCTGACAGTTG\tCCGGACTACHVGGGTWTCTAAT\tsample2"]
 
 
 #run unit tests if run from command-line
